package com.tcctyn.system.api.factory;

import com.tcctyn.system.api.domain.SysDept;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cloud.openfeign.FallbackFactory;
import org.springframework.stereotype.Component;
import com.tcctyn.common.core.web.domain.R;
import com.tcctyn.system.api.RemoteUserService;
import com.tcctyn.system.api.domain.SysUser;
import com.tcctyn.system.api.model.LoginUser;

import java.util.List;

/**
 * 用户服务降级处理
 * 
 * @author tcctyn
 */
@Component
public class RemoteUserFallbackFactory implements FallbackFactory<RemoteUserService>
{
    private static final Logger log = LoggerFactory.getLogger(RemoteUserFallbackFactory.class);

    @Override
    public RemoteUserService create(Throwable throwable)
    {
        log.error("用户服务调用失败:{}", throwable.getMessage());
        return new RemoteUserService()
        {
            @Override
            public R<LoginUser> getUserInfo(String username, String source)
            {
                return R.fail("获取用户失败:" + throwable.getMessage());
            }

            @Override
            public R<Boolean> registerUserInfo(SysUser sysUser, String source)
            {
                return R.fail("注册用户失败:" + throwable.getMessage());
            }

            @Override
            public R<Boolean> recordUserLogin(SysUser sysUser, String source)
            {
                return R.fail("记录用户登录信息失败:" + throwable.getMessage());
            }

            @Override
            public R<SysUser> selectUserById(Long userId) {
                return R.fail("获取用户信息失败:" + throwable.getMessage());
            }

            @Override
            public R<List<SysUser>> selectUserList(SysUser sysUser) {
                return R.fail("获取用户信息列表失败:" + throwable.getMessage());
            }

            @Override
            public R<List<SysUser>> selectUserByRegionIdList(Long regionId) {
                return R.fail("获取用户区域id失败:" + throwable.getMessage());
            }

            @Override
            public R<SysDept> selectDeptById(Long deptId) {
                return R.fail("获取部门信息失败:" + throwable.getMessage());
            }

            @Override
            public R<List<SysDept>> selectDeptList(SysDept sysDept) {
                return R.fail("获取部门信息列表失败:" + throwable.getMessage());
            }
        };
    }
}
