package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.ApprovalRecordVO;
import com.tcctyn.iot.forestfire.domain.ApprovalRecord;
import com.tcctyn.iot.forestfire.searcher.approvalrecord.ApprovalRecordRejectRequest;
import com.tcctyn.iot.forestfire.service.IApprovalRecordService;
import com.tcctyn.iot.forestfire.service.ICommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 * 审批记录表 前端控制器
 * </p>
 *
 * @author daichao
 * @since 2024-12-09
 */


@RestController
@RequestMapping("/approvalRecord")
public class ApprovalRecordController extends BaseController {


    @Autowired
    private IApprovalRecordService iApprovalRecordService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<ApprovalRecord> pageList = iApprovalRecordService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        ApprovalRecord approvalRecord = iApprovalRecordService.getById(id);
        ApprovalRecordVO approvalRecordVO = new ApprovalRecordVO();
        BeanUtils.copyBeanProp(approvalRecord, approvalRecordVO);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(approvalRecordVO.getReviewerId());
        approvalRecordVO.setReviewerName(Optional.ofNullable(userIdToNameMap.get(approvalRecordVO.getReviewerId())).orElse(""));
        return AjaxResult.success(approvalRecordVO);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody ApprovalRecord params) {
        Integer result = iApprovalRecordService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iApprovalRecordService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody ApprovalRecord params) {
        Integer result = iApprovalRecordService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 审批记录分页查询
     *
     * @param approvalRecord
     * @return
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(ApprovalRecord approvalRecord) {
        startPage();
        List<ApprovalRecord> list = iApprovalRecordService.selectList(approvalRecord);
        List<ApprovalRecordVO> voList = BeanUtils.copyList(list, ApprovalRecordVO.class);
        //映射给前端需要的审批人姓名
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        voList.forEach(item -> item.setReviewerName(Optional.ofNullable(userIdToNameMap.get(item.getReviewerId())).orElse("")));
        return getDataTable(list, voList);
    }

    /**
     * 审批通过接口
     *
     * @param id
     * @return
     */
    @PostMapping(value = "/approved/{id}")
    public AjaxResult approved(@PathVariable("id") Long id) {
//        Long userId = 1L;
        Long userId = super.getUserId();
        Integer result = iApprovalRecordService.approved(id, userId);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/reject")
    public AjaxResult reject(@Validated @RequestBody ApprovalRecordRejectRequest request) {
//        Long userId = 1L;
        Long userId = super.getUserId();
        Integer result = iApprovalRecordService.reject(request, userId);
        return AjaxResult.success(result);
    }
}
