package com.tcctyn.iot.forestfire.controller;

import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.CheckInRecordVO;
import com.tcctyn.iot.forestfire.domain.CheckInRecord;
import com.tcctyn.iot.forestfire.service.ICheckInRecordService;
import com.tcctyn.iot.forestfire.service.ICommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 * 打卡记录表 前端控制器
 * </p>
 *
 * @author liuzh
 * @since 2024-10-12
 */

@RestController
@RequestMapping("/checkInRecord")
public class CheckInRecordController extends BaseController {


    @Autowired
    private ICheckInRecordService iCheckInRecordService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public TableDataInfo list(CheckInRecord checkInRecord) {
        startPage();
        List<CheckInRecord> list = iCheckInRecordService.list(checkInRecord);
        return getDataTable(list);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        CheckInRecord checkInRecord = iCheckInRecordService.getById(id);
        return AjaxResult.success(checkInRecord);
    }

    /**
     * APP端打卡，生成打卡记录
     */
    @PostMapping(value = "/create")
    public AjaxResult create(@Validated @RequestBody CheckInRecord checkInRecord) {
//        Long userId = 102L;
        Long userId = super.getUserId();
        Integer result = iCheckInRecordService.create(checkInRecord, userId);
        return AjaxResult.success(result);
    }

    /**
     * PC端查看最近十条打卡上报记录
     */
    @GetMapping(value = "/selectLast10Items")
    public AjaxResult selectLast10Items() {
        List<CheckInRecord> list = iCheckInRecordService.selectLast10Items();
        //组装护林员姓名
        List<CheckInRecordVO> voList = BeanUtils.copyList(list, CheckInRecordVO.class);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        voList.forEach(item -> item.setUserName(Optional.ofNullable(userIdToNameMap.get(item.getUserId())).orElse("")));
        return AjaxResult.success(voList);
    }


}
