package com.tcctyn.iot.forestfire.controller;

import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.iot.forestfire.VO.DeviceChanelsVO;
import com.tcctyn.iot.forestfire.domain.DeviceChanels;
import com.tcctyn.iot.forestfire.service.IDeviceChanelsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/deviceChanels")
public class DeviceChanelsController extends BaseController {

    @Autowired
    private IDeviceChanelsService deviceChanelsService;
    @GetMapping("/getById")
    public AjaxResult getById(@RequestParam Long id) {
        DeviceChanelsVO deviceChanelsVO = deviceChanelsService.getById(id);
        return AjaxResult.success(deviceChanelsVO);
    }

    @PostMapping("/create")
    public AjaxResult create(@RequestBody DeviceChanels deviceChanels) {
        Integer result = deviceChanelsService.create(deviceChanels);
        return AjaxResult.success(result);
    }

    @DeleteMapping("/delete")
    public AjaxResult delete(@RequestParam Long id) {
        Integer result = deviceChanelsService.delete(id);
        return AjaxResult.success(result);
    }

    @PutMapping("/update")
    public AjaxResult update(@RequestBody DeviceChanels deviceChanels) {
        Integer result = deviceChanelsService.update(deviceChanels);
        return AjaxResult.success(result);
    }

   /**
    * 根据设备编号查询设备通道信息
    */
    @GetMapping("/selectByDeviceNo")
    public AjaxResult selectByDeviceNo(@RequestParam String deviceNo) {
        List<DeviceChanelsVO> vo = deviceChanelsService.selectByDeviceNo(deviceNo);
        return AjaxResult.success(vo);
    }

    /**
    * 根据设备编号和通道号查询设备通道信息
    */
    @GetMapping("/getStreamUrl")
    public AjaxResult getStreamUrl(@RequestParam String deviceNo, @RequestParam String chanelNo) {
        DeviceChanelsVO vo = deviceChanelsService.getStreamUrl(deviceNo, chanelNo);
        String result = vo.getStreamUrl();
        return AjaxResult.success(result);
    }

    /**
    * 预览
    */
    @GetMapping("/startPreview")
    public AjaxResult startPreview(@RequestParam String deviceNo, @RequestParam String chanelNo) {
        DeviceChanelsVO vo = deviceChanelsService.startPreview(deviceNo, chanelNo);
        String result = vo.getStreamUrl();
        return AjaxResult.success(result);
    }

    /**
    * 停止预览
    */
    @GetMapping("/stopPreview")
    public AjaxResult stopPreview(@RequestParam String deviceNo, @RequestParam String chanelNo) {
        String vo = deviceChanelsService.stopPreview(deviceNo, chanelNo);
        return AjaxResult.success(vo);
    }
}
