package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.DroneTaskPlanVO;
import com.tcctyn.iot.forestfire.domain.DroneTaskPlan;
import com.tcctyn.iot.forestfire.enums.DroneTaskStatusEnum;
import com.tcctyn.iot.forestfire.searcher.drone.DroneSearcher;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IDroneTaskPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 无人机任务计划表 前端控制器
 * </p>
 *
 * @author 代超
 * @since 2024-11-07
 */

@RestController
@RequestMapping("/droneTaskPlan")
public class DroneTaskPlanController extends BaseController {


    @Autowired
    private IDroneTaskPlanService iDroneTaskPlanService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<DroneTaskPlan> pageList = iDroneTaskPlanService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        DroneTaskPlan droneTaskPlan = iDroneTaskPlanService.getById(id);
        return AjaxResult.success(droneTaskPlan);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody DroneTaskPlan params) {
        Integer result = iDroneTaskPlanService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iDroneTaskPlanService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody DroneTaskPlan params) {
        Integer result = iDroneTaskPlanService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 大屏页面查询无人机的不同状态的任务列表
     *
     * @param searcher
     * @return
     */
    @PostMapping(value = "/selectByStatus")
    public AjaxResult selectByStatus(@RequestBody DroneSearcher searcher) {
        List<DroneTaskPlan> droneTaskPlanList = iDroneTaskPlanService.selectByStatus(searcher);
        List<DroneTaskPlanVO> voList = BeanUtils.copyList(droneTaskPlanList, DroneTaskPlanVO.class);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(droneTaskPlanVO -> {
            droneTaskPlanVO.setRegionName(regionIdToNameMap.get(droneTaskPlanVO.getRegionId()) == null ? "" : regionIdToNameMap.get(droneTaskPlanVO.getRegionId()));
        });
        return AjaxResult.success(voList);
    }

    /**
     * 分页查询无人机的历史执飞记录
     */
    @GetMapping(value = "/listHistory")
    public TableDataInfo listHistory(DroneTaskPlan droneTaskPlan) {
        droneTaskPlan.setTaskStatus(DroneTaskStatusEnum.COMPLETED.name());
        startPage();
        List<DroneTaskPlan> droneTaskPlanList = iDroneTaskPlanService.seletList(droneTaskPlan);
        List<DroneTaskPlanVO> voList = BeanUtils.copyList(droneTaskPlanList, DroneTaskPlanVO.class);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(droneTaskPlanVO -> {
            droneTaskPlanVO.setRegionName(regionIdToNameMap.get(droneTaskPlanVO.getRegionId()) == null ? "" : regionIdToNameMap.get(droneTaskPlanVO.getRegionId()));
        });
        return getDataTable(droneTaskPlanList, voList);
    }
}
