package com.tcctyn.iot.forestfire.controller;

import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.iot.forestfire.enums.*;
import com.tcctyn.iot.forestfire.searcher.EnumSearcher;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 枚举类的公共接口
 */
@RestController
@RequestMapping("/enum")
public class EnumController extends BaseController {


    private final static Map<String, Map<String, String>> type2EnumName2DescMap = new LinkedHashMap<>();

    /**
     * 前端需要的枚举信息需要在此处初始化
     */
    static {
        type2EnumName2DescMap.put("AreaTypeEnum", AreaTypeEnum.getName2DescMap());
        type2EnumName2DescMap.put("PointTypeEnum", PointTypeEnum.getName2DescMap());
        type2EnumName2DescMap.put("FireWarningDisposalStatusEnum", FireWarningDisposalStatusEnum.getName2DescMap());
        type2EnumName2DescMap.put("DeviceStatusEnum", DeviceStatusEnum.getName2DescMap());
        type2EnumName2DescMap.put("DataSourceEnum", DataSourceEnum.getName2DescMap());
        type2EnumName2DescMap.put("TaskUrgencyEnum", TaskUrgencyEnum.getName2DescMap());
        type2EnumName2DescMap.put("TaskStatusEnum", TaskStatusEnum.getName2DescMap());
        type2EnumName2DescMap.put("ReportMethodEnum", ReportMethodEnum.getName2DescMap());
        type2EnumName2DescMap.put("FireHazardLevelEnum", FireHazardLevelEnum.getName2DescMap());
        type2EnumName2DescMap.put("DroneTaskStatusEnum", DroneTaskStatusEnum.getName2DescMap());
        type2EnumName2DescMap.put("ApprovalBusinessTypeEnum", ApprovalBusinessTypeEnum.getName2DescMap());
    }

    @Tag(name = "按需查询枚举接口",description = "按需查询枚举接口")
    @PostMapping(value = "queryEnum")
    public AjaxResult queryEnum(@RequestBody EnumSearcher enumSearcher) {
        //对枚举值进行校验
        if (CollectionUtils.isEmpty(enumSearcher.getEnumNameList())) {
            return error("查询列表为空，请检查！");
        }
        for (String enumName : enumSearcher.getEnumNameList()) {
            if (!type2EnumName2DescMap.containsKey(enumName)) {
                return error("枚举类型参数非法");
            }
        }

        final Map<String, Map<String, String>> resultMap = new LinkedHashMap<>(enumSearcher.getEnumNameList().size());
        for (String enumType : enumSearcher.getEnumNameList()) {
            resultMap.put(enumType, type2EnumName2DescMap.get(enumType));
        }

        return AjaxResult.success(resultMap);
    }
}
