package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.LeaveRecordVO;
import com.tcctyn.iot.forestfire.domain.LeaveRecord;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.ILeaveRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 * 请假记录表 前端控制器
 * </p>
 *
 * @author daichao
 * @since 2024-12-09
 */

@RestController
@RequestMapping("/leaveRecord")
public class LeaveRecordController extends BaseController {


    @Autowired
    private ILeaveRecordService iLeaveRecordService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<LeaveRecord> pageList = iLeaveRecordService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        LeaveRecord leaveRecord = iLeaveRecordService.getById(id);
        LeaveRecordVO leaveRecordVO = new LeaveRecordVO();
        BeanUtils.copyBeanProp(leaveRecord, leaveRecordVO);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        Map<Long, String> deptIdToNameMap = commonService.deptMapIdToName(leaveRecordVO.getDeptId());
        leaveRecordVO.setUserName(Optional.ofNullable(userIdToNameMap.get(leaveRecordVO.getUserId())).orElse(""));
        leaveRecordVO.setDeptName(Optional.ofNullable(deptIdToNameMap.get(leaveRecordVO.getDeptId())).orElse(""));
        leaveRecordVO.setReviewerName(Optional.ofNullable(userIdToNameMap.get(leaveRecordVO.getReviewerId())).orElse(""));
        return AjaxResult.success(leaveRecordVO);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@Validated @RequestBody LeaveRecord params) {
        //TODO 登录用户的userId 跟deptId 是可以拿到的，前端无需传
//        Long userId = 100L;
//        Long deptId = 100L;
        Long userId = super.getUserId();
        Long deptId = super.getDeptId();
        params.setUserId(userId);
        params.setDeptId(deptId);
        Integer result = iLeaveRecordService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iLeaveRecordService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody LeaveRecord params) {
        Integer result = iLeaveRecordService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 请假记录分页查询接口
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(LeaveRecord leaveRecord) {
//        Long userId = 100L;
        Long userId = super.getUserId();
        leaveRecord.setUserId(userId);
        startPage();
        List<LeaveRecord> list = iLeaveRecordService.selectList(leaveRecord);
        List<LeaveRecordVO> voList = BeanUtils.copyList(list, LeaveRecordVO.class);
        //此处需要映射给前端请假人姓名，部门名称，审批人姓名
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        Map<Long, String> deptIdToNameMap = commonService.deptMapIdToName(null);
        voList.forEach(item -> {
            item.setUserName(Optional.ofNullable(userIdToNameMap.get(item.getUserId())).orElse(""));
            item.setDeptName(Optional.ofNullable(deptIdToNameMap.get(item.getDeptId())).orElse(""));
            item.setReviewerName(Optional.ofNullable(userIdToNameMap.get(item.getReviewerId())).orElse(""));
        });
        return getDataTable(list, voList);
    }

    /**
     * 请假记录撤销接口
     *
     * @param id
     * @return
     */
    @PostMapping(value = "/revoke/{id}")
    public AjaxResult revoke(@PathVariable("id") Long id) {
        Integer result = iLeaveRecordService.revoke(id);
        return AjaxResult.success(result);
    }
}
