package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.NoticeInfoVO;
import com.tcctyn.iot.forestfire.domain.NoticeInfo;
import com.tcctyn.iot.forestfire.searcher.noticeinfo.NoticeEnableFlagRequest;
import com.tcctyn.iot.forestfire.service.INoticeInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 通知公告信息表 前端控制器
 * </p>
 *
 * @author daichao
 * @since 2024-12-17
 */

@RestController
@RequestMapping("/noticeInfo")
public class NoticeInfoController extends BaseController {


    @Autowired
    private INoticeInfoService iNoticeInfoService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<NoticeInfo> pageList = iNoticeInfoService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        NoticeInfo noticeInfo = iNoticeInfoService.getById(id);
        NoticeInfoVO noticeInfoVO = new NoticeInfoVO();
        BeanUtils.copyBeanProp(noticeInfo, noticeInfoVO);
        return AjaxResult.success(noticeInfoVO);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody NoticeInfo params) {
        Integer result = iNoticeInfoService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iNoticeInfoService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@Validated @RequestBody NoticeInfo params) {
        Integer result = iNoticeInfoService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 分页列表查询
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(NoticeInfo noticeInfo) {
        startPage();
        List<NoticeInfo> noticeInfoList = iNoticeInfoService.selectList(noticeInfo);
        List<NoticeInfoVO> voList = BeanUtils.copyList(noticeInfoList, NoticeInfoVO.class);
        return getDataTable(noticeInfoList, voList);
    }

    /**
     * APP端获取需要展示的通知公告列表
     */
    @GetMapping(value = "/selectEnabled")
    public AjaxResult selectEnabled() {
        NoticeInfo noticeInfo = new NoticeInfo();
        noticeInfo.setEnableFlag(1);
        List<NoticeInfo> noticeInfoList = iNoticeInfoService.selectList(noticeInfo);
        List<NoticeInfoVO> voList = BeanUtils.copyList(noticeInfoList, NoticeInfoVO.class);
        return AjaxResult.success(voList);
    }

    /**
     * 通知公告启用/禁用接口
     */
    @PostMapping(value = "/updateEnableFlag")
    public AjaxResult updateEnableFlag(@Validated @RequestBody NoticeEnableFlagRequest request) {
        NoticeInfo noticeInfo = new NoticeInfo();
        BeanUtils.copyBeanProp(request, noticeInfo);
        Integer result = iNoticeInfoService.update(noticeInfo);
        return AjaxResult.success(result);
    }
}
