package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.PatrolPathVO;
import com.tcctyn.iot.forestfire.domain.PatrolPath;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IPatrolPathService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 巡护路径表 前端控制器
 * </p>
 *
 * @author 代超
 * @since 2024-09-24
 */

@RestController
@RequestMapping("/patrolPath")
public class PatrolPathController extends BaseController {


    @Autowired
    private IPatrolPathService iPatrolPathService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<PatrolPath> pageList = iPatrolPathService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "get/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        PatrolPathVO patrolPathVO = iPatrolPathService.getById(id);
        return AjaxResult.success(patrolPathVO);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@Validated @RequestBody PatrolPath params) {
//        params.setCreatedBy(getUsername());
        Integer result = iPatrolPathService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iPatrolPathService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@Validated @RequestBody PatrolPath params) {
//        params.setUpdatedBy(getUsername());
        Integer result = iPatrolPathService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 巡护路径分页查询
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(PatrolPath patrolPath) {
        startPage();
        List<PatrolPath> list = iPatrolPathService.selectList(patrolPath);
        //考虑到分页的实现，DO->VO 这步放在controller层实现
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        List<PatrolPathVO> voList = BeanUtils.copyList(list, PatrolPathVO.class);
        voList.forEach(patrolPathVO -> {
            String forestName = regionIdToNameMap.get(patrolPathVO.getForestId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getForestId());
            String farmName = regionIdToNameMap.get(patrolPathVO.getFarmId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getFarmId());
            String fenceName = regionIdToNameMap.get(patrolPathVO.getFenceId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getFenceId());
            patrolPathVO.setBelongingRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
        });
        return getDataTable(list,voList);
    }

    /**
     * 根据林场id查询巡护路径
     *
     * @param regionId
     * @return
     */
    @GetMapping(value = "selectByRegionId/{regionId}")
    public AjaxResult selectByRegionId(@PathVariable("regionId") Long regionId) {
        List<PatrolPath> patrolPathList = iPatrolPathService.selectByRegionId(regionId);
        return AjaxResult.success(patrolPathList);
    }

    /**
     * 大屏页面巡护路径图标接口，
     * 根据林场、林区、电子围栏查询当下的所有巡护路径
     *
     * @param patrolPath
     * @return
     */
    @GetMapping(value = "queryByRegion")
    public AjaxResult queryByRegion(PatrolPath patrolPath) {
        List<PatrolPathVO> list = iPatrolPathService.queryByRegion(patrolPath);
        return AjaxResult.success(list);
    }
}
