package com.tcctyn.iot.forestfire.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.DeviceUnderPointVO;
import com.tcctyn.iot.forestfire.VO.PointAndDeviceInfoVO;
import com.tcctyn.iot.forestfire.VO.PointInfoVO;
import com.tcctyn.iot.forestfire.VO.PointStatisticsByPointTypeVO;
import com.tcctyn.iot.forestfire.domain.DeviceInfo;
import com.tcctyn.iot.forestfire.domain.PointInfo;
import com.tcctyn.iot.forestfire.enums.DeviceStatusEnum;
import com.tcctyn.iot.forestfire.searcher.BaseIdListSearcher;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IDeviceInfoService;
import com.tcctyn.iot.forestfire.service.IPointInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 点位信息表 前端控制器
 * </p>
 *
 * @author 代超
 * @since 2024-09-24
 */

@RestController
@RequestMapping("/pointInfo")
public class PointInfoController extends BaseController {


    @Autowired
    private IPointInfoService iPointInfoService;

    @Autowired
    private ICommonService commonService;

    @Autowired
    private IDeviceInfoService deviceInfoService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<PointInfo> pageList = iPointInfoService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/get/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        PointInfoVO vo = iPointInfoService.getById(id);
        return AjaxResult.success(vo);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@Validated @RequestBody PointInfo params) {
//        params.setCreatedBy(getUsername());
        Integer result = iPointInfoService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iPointInfoService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@Validated @RequestBody PointInfo params) {
//        params.setUpdatedBy(getUsername());
        Integer result = iPointInfoService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 设备快捷修改启用/停用状态接口
     *
     * @param params
     * @return
     */
    @PostMapping(value = "/updateEnableFlag")
    public AjaxResult updateEnableFlag(@RequestBody PointInfo params) {
        Integer result = iPointInfoService.updateEnableFlag(params);
        return AjaxResult.success(result);
    }//sgq2025.3.20

    /**
     * 条件查询，不分页，可用于查询云台、卡口
     *
     * @param pointInfo
     * @return
     */
    @GetMapping(value = "/selectAll")
    public AjaxResult selectAll(PointInfo pointInfo) {
        List<PointInfo> list = iPointInfoService.selectAll(pointInfo);
        return AjaxResult.success(list);
    }

    /**
     * 点位批量删除接口
     *
     * @param searcher
     * @return
     */
    @PostMapping(value = "/deleteBatch")
    public AjaxResult deleteBatch(@RequestBody BaseIdListSearcher<Long> searcher) {
        Integer result = iPointInfoService.deleteBatch(searcher.getIdList());
        return AjaxResult.success(result);
    }

    /**
     * 资源点分页查询
     *
     * @param pointInfo
     * @return
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(PointInfo pointInfo) {
        startPage();
        List<PointInfo> list = iPointInfoService.selectList(pointInfo);
        //处理归属地名称
        List<PointInfoVO> voList = BeanUtils.copyList(list, PointInfoVO.class);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);

        //需要加上点位下的设备信息：设备总数、在线设备数、离线设备数;在分页的基础上去关联统计设备信息，减少查询的数据量
        List<Long> pointIdList = voList.stream().map(PointInfoVO::getId).collect(Collectors.toList());
        Map<Long, DeviceUnderPointVO> pointAndDeviceCountMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(pointIdList)) {
            List<DeviceInfo> deviceInfoList = deviceInfoService.selectListByPointList(pointIdList);
            Map<Long, List<DeviceInfo>> pointIdAndDeviceInfoListMap = deviceInfoList.stream().collect(Collectors.groupingBy(DeviceInfo::getPointId));
            pointIdAndDeviceInfoListMap.forEach((key, value) -> {
                DeviceUnderPointVO deviceUnderPointVO = new DeviceUnderPointVO();
                deviceUnderPointVO.setTotalNum(value.size());
                deviceUnderPointVO.setOnlineNum(value.stream().filter(item -> DeviceStatusEnum.ON.name().equals(item.getDeviceStatus())).collect(Collectors.toList()).size());
                deviceUnderPointVO.setOfflineNum(deviceUnderPointVO.getTotalNum() - deviceUnderPointVO.getOnlineNum());
                pointAndDeviceCountMap.put(key, deviceUnderPointVO);
            });
        }
        voList.forEach(pointInfoVO -> {
            String forestName = regionIdToNameMap.get(pointInfoVO.getForestId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getForestId());
            String farmName = regionIdToNameMap.get(pointInfoVO.getFarmId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getFarmId());
            String fenceName = regionIdToNameMap.get(pointInfoVO.getFenceId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getFenceId());
            pointInfoVO.setBelongingRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
            pointInfoVO.setDeviceUnderPointVO(pointAndDeviceCountMap.get(pointInfoVO.getId()) == null ? new DeviceUnderPointVO() : pointAndDeviceCountMap.get(pointInfoVO.getId()));
        });
        return getDataTable(list, voList);
    }

    /**
     * 大屏页面查询监测点信息
     * 查询云台卡口信息，将点位类型写死为 云台、卡口
     *
     * @param pointInfo
     * @return
     */
    @GetMapping(value = "/queryMonitoringPoint")
    public AjaxResult queryMonitoringPoint(PointInfo pointInfo) {
        List<PointAndDeviceInfoVO> list = iPointInfoService.queryMonitoringPoint(pointInfo);
        return AjaxResult.success(list);
    }

    /**
     * 资源点分类统计接口
     *
     * @param
     * @return
     */
    @PostMapping(value = "/statisticsByPointType")
    public AjaxResult statisticsByPointType() {
        PointStatisticsByPointTypeVO vo = iPointInfoService.statisticsByPointType();
        return AjaxResult.success(vo);
    }
}
