package com.tcctyn.iot.forestfire.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.iot.forestfire.service.IRegionInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 区域信息表 前端控制器
 * </p>
 *
 * @author 代超
 * @since 2024-09-24
 */


@RestController
@RequestMapping("/regionInfo")
public class RegionInfoController extends BaseController {


    @Autowired
    private IRegionInfoService iRegionInfoService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<RegionInfo> pageList = iRegionInfoService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/get/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        RegionInfo regionInfo = iRegionInfoService.getById(id);
        return AjaxResult.success(regionInfo);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@Validated @RequestBody RegionInfo params) {
//        params.setCreatedBy(getUsername());
        Integer result = iRegionInfoService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iRegionInfoService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@Validated @RequestBody RegionInfo params) {
//        params.setUpdatedBy(getUsername());
        Integer result = iRegionInfoService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 林区分页查询
     *
     * @param regionInfo
     * @return
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(RegionInfo regionInfo) {
        startPage();
        List<RegionInfo> list = iRegionInfoService.selectList(regionInfo);
        return getDataTable(list);
    }

    /**
     * 根据父级id查询子级区域信息
     *
     * @param parentId
     * @return
     */
    @GetMapping(value = "/selectAllByParentId/{parentId}")
    public AjaxResult selectAllByParentId(@PathVariable("parentId") Long parentId) {
        List<RegionInfo> list = iRegionInfoService.selectAllByParentId(parentId);
        return AjaxResult.success(list);
    }

    /**
     * 查询完整的区域树信息
     *
     * @param
     * @return
     */
    @GetMapping(value = "/selectAll/{level}")
    public TableDataInfo selectAll(@PathVariable("level") Integer level) {
        List<RegionInfo> list = iRegionInfoService.selectAll(level);
        return getDataTable(list);
    }

    /**
     * 根据地区类型查询区域信息
     *
     * @param areaType
     * @return
     */
    @GetMapping(value = "/selectAllByAreaType/{areaType}")
    public AjaxResult selectAllByAreaType(@PathVariable("areaType") String areaType) {
        List<RegionInfo> list = iRegionInfoService.selectAllByAreaType(areaType);
        return AjaxResult.success(list);
    }

    /**
     * 查询树形区域信息
     *
     * @param
     * @return
     */
    @GetMapping(value = "/regionTree")
    public AjaxResult regionTree(RegionInfo regionInfo) {
        return success(iRegionInfoService.selectRegionTree(regionInfo));
    }
}
