package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.iot.forestfire.VO.AirportVO;
import com.tcctyn.iot.forestfire.domain.SatelliteDataSimulation;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.ISatelliteDataSimulationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 卫星数据模拟表 前端控制器
 * </p>
 *
 * @author 代超
 * @since 2024-11-05
 */

@RestController
@RequestMapping("/satelliteDataSimulation")
public class SatelliteDataSimulationController extends BaseController {


    @Autowired
    private ISatelliteDataSimulationService iSatelliteDataSimulationService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<SatelliteDataSimulation> pageList = iSatelliteDataSimulationService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        SatelliteDataSimulation satelliteDataSimulation = iSatelliteDataSimulationService.getById(id);
        return AjaxResult.success(satelliteDataSimulation);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody SatelliteDataSimulation params) {
        Integer result = iSatelliteDataSimulationService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iSatelliteDataSimulationService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody SatelliteDataSimulation params) {
        Integer result = iSatelliteDataSimulationService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 查询最近一批十五分钟内的数据
     *
     * @param
     * @return
     */
    @GetMapping(value = "/selectRecentlyData")
    public AjaxResult selectRecentlyData(@RequestParam Date currentTime) {
        List<SatelliteDataSimulation> list = iSatelliteDataSimulationService.selectRecentlyData(currentTime);
        return AjaxResult.success(list);
    }

    /**
     * 对某个高风险地区进行降风险处理,
     * 需要返回给前端最近的机场的坐标，方便前端模拟无人机起飞的效果
     */
    @PostMapping(value = "/reduceRisk/{id}")
    public AjaxResult reduceRisk(@PathVariable("id") Long id) {
        AirportVO vo = iSatelliteDataSimulationService.reduceRisk(id);
        Map<Long, String> regionIdAndNameMap = commonService.regionMapIdToName(null);
        vo.setRegionName(regionIdAndNameMap.get(vo.getRegionId()));
        return AjaxResult.success(vo);
    }
}
