package com.tcctyn.iot.forestfire.controller;

import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.SchedulePlanVO;
import com.tcctyn.iot.forestfire.domain.SchedulePlan;
import com.tcctyn.iot.forestfire.service.IRegionInfoService;
import com.tcctyn.iot.forestfire.service.ISchedulePlanService;
import com.tcctyn.iot.forestfire.service.IScheduleTemplateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 排班计划 前端控制器
 * </p>
 *
 * @author liuzh
 * @since 2024-10-16
 */

@RestController
@RequestMapping("/schedulePlan")
public class SchedulePlanController extends BaseController {

    @Autowired
    private ISchedulePlanService iSchedulePlanService;

    @Autowired
    private IScheduleTemplateService iScheduleTemplateService;

    @Autowired
    private IRegionInfoService regionInfoService;

    @GetMapping(value = "/list")
    public TableDataInfo list(SchedulePlan schedulePlan) {
        startPage();
        List<SchedulePlan> doList = iSchedulePlanService.list(schedulePlan);
        List<SchedulePlanVO> voList = new ArrayList<>();
        doList.forEach(schedulePlan1 -> {
            SchedulePlanVO schedulePlanVo = new SchedulePlanVO();
            //如果 forestId, farmId, fenceId 有值，就查对应的区域名称并拼接到 regionName 中
            String regionName = "";
            if (schedulePlan1.getForestId() != null) {
                regionName += regionInfoService.getById(schedulePlan1.getForestId()).getAreaName() + ",";
            }
            if (schedulePlan1.getFarmId() != null) {
                regionName += regionInfoService.getById(schedulePlan1.getFarmId()).getAreaName() + ",";
            }
            if (schedulePlan1.getFenceId() != null) {
                regionName += regionInfoService.getById(schedulePlan1.getFenceId()).getAreaName();
            }
            BeanUtils.copyBeanProp(schedulePlan1, schedulePlanVo);
            schedulePlanVo.setRegionName(regionName);;
            voList.add(schedulePlanVo);
        });

        return getDataTable(doList, voList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        SchedulePlan schedulePlan = iSchedulePlanService.getById(id);
        return AjaxResult.success(schedulePlan);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody SchedulePlan params) {
//        params.setCreatedBy(getUsername());
        Integer result = iSchedulePlanService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result =  iSchedulePlanService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody SchedulePlan params) {
//        params.setUpdatedBy(getUsername());
        Integer result = iSchedulePlanService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 根据模板生成排班信息
     * @param params
     * @return
     */
    @PostMapping(value = "/generate")
    public AjaxResult generate(@RequestBody SchedulePlan params) {

        Integer result = iSchedulePlanService.generate(params);
        return AjaxResult.success(result);
    }
}
