package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.DateUtils;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.*;
import com.tcctyn.iot.forestfire.domain.CheckInRecord;
import com.tcctyn.iot.forestfire.domain.PointInfo;
import com.tcctyn.iot.forestfire.domain.Schedules;
import com.tcctyn.iot.forestfire.mapper.PointInfoMapper;
import com.tcctyn.iot.forestfire.searcher.schedules.ScheduleSwapSearcher;
import com.tcctyn.iot.forestfire.searcher.schedules.SchedulesSearcher;
import com.tcctyn.iot.forestfire.service.*;
import com.tcctyn.system.api.RemoteUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.YearMonth;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 排班表 前端控制器
 * </p>
 *
 * @author liuzh
 * @since 2024-09-24
 */

@RestController
@RequestMapping("/schedules")
public class SchedulesController extends BaseController {

    @Autowired
    private ISchedulesService iSchedulesService;

    @Autowired
    private IPatrolPathService patrolPathService;

    @Resource
    private RemoteUserService remoteUserService;

    @Autowired
    private ICheckInRecordService checkInRecordService;

    @Resource
    private PointInfoMapper pointInfoMapper;

    @Autowired
    private ISchedulePlanService schedulePlanService;

    @Autowired
    private ICommonService commonService;

    /**
     * 获取周排班信息
     *
     * @param schedules
     * @return
     */
    @GetMapping(value = "/list")
    public AjaxResult list(Schedules schedules) {
        AjaxResult ajax = AjaxResult.success();
        //如果日期为空，则设置为当前日期
        if (schedules.getDateOfWeek() == null) {
            schedules.setDateOfWeek(new Date());
        }
        List<ScheduleDetail> weeklist = iSchedulesService.weekList(schedules);
//        startPage();
        List<Map<String, Object>> list = iSchedulesService.list(weeklist, schedules);
        ajax.put("rows", list);
        List<String> dateList = new ArrayList<>(7);
        Date currentMonday = DateUtils.getThisWeekMonday(schedules.getDateOfWeek());
        for (int i = 0; i < 7; i++) {
            dateList.add(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, DateUtils.addDays(currentMonday, i)));
        }
        ajax.put("dateList", dateList);
        return ajax;
    }

    /**
     * 生成下周的排班信息
     *
     * @return
     */
    /*@PostMapping(value = "/generate")
    public AjaxResult generate() {
        if (iSchedulesService.generate()) {
            return AjaxResult.success();
        } else {
            return AjaxResult.warn("生成失败");
        }
    }*/
    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        Schedules schedules = iSchedulesService.getById(id);
        return AjaxResult.success(schedules);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody Schedules params) {
        Integer result = iSchedulesService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iSchedulesService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody Schedules params) {
        Integer result = iSchedulesService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 批量操作
     *
     * @param
     * @return
     */
    /*@PostMapping(value = "/batch")
    public AjaxResult createBatch(@RequestBody ScheduleDetail scheduleDetail) {
        if (scheduleDetail.getDateOfWeek().isEmpty()) {
            return AjaxResult.warn("日期不能为空");

        }
        Integer result = iSchedulesService.createBatch(scheduleDetail);
        return AjaxResult.success(result);
    }*/

    /**
     * 查询列表
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(Schedules schedules,
                                    @RequestParam(required = false) Date startDate,
                                    @RequestParam(required = false) Date endDate,
                                    @RequestParam(required = false) String userName) {
        startPage();
        List<Schedules> doList = iSchedulesService.selectList(schedules, startDate, endDate, userName);
        List<ScheduleVO> voList = new ArrayList<>();
        for (Schedules doItem : doList) {
            ScheduleVO voItem = new ScheduleVO();
            BeanUtils.copyProperties(doItem, voItem);
            voItem.setUserName(remoteUserService.selectUserById(doItem.getUserId()).getData().getNickName());
            voItem.setPathName(commonService.pathMapIdToName(doItem.getPathId()).get(doItem.getPathId()));
            //增加今日打卡总共需要打卡次数
            List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().eq("belonging_region_id", doItem.getPathId()).eq("point_type", "CHECK_IN_POINT"));
            voItem.setTotalTimes(pointInfoList.size());
            //增加今日打卡次数
            List<CheckInRecord> currentCheckInRecord = checkInRecordService.getByUserId(doItem.getUserId(), doItem.getDateOfWeek());
            if (!currentCheckInRecord.isEmpty()) {
                voItem.setCheckTimes(currentCheckInRecord.size());
            } else {
                voItem.setCheckTimes(0);
            }

            voList.add(voItem);
        }
        return getDataTable(doList, voList);
    }


    /**
     * 单个用户调班
     *
     * @param params
     * @return
     */
    @PostMapping(value = "/swap")
    public AjaxResult swap(@RequestBody ScheduleSwapSearcher params) {
        Integer result = iSchedulesService.swapSchedule(params);
        return AjaxResult.success(result);
    }

    /**
     * 大屏页面按钮，展示当日的人员巡护信息及打卡信息
     * 需要暂时当日值班人员的巡护路径、最新打卡点位置信息、无打卡信息的人员位置信息默认用路径的第一个点位经纬度作为人员的当前位置
     */
    @PostMapping(value = "/queryRangerInfo")
    public AjaxResult queryRangerInfo(@RequestBody(required = false) SchedulesSearcher params) {
        List<RangerInfoVO> voList = iSchedulesService.queryRangerInfo(params);
        return AjaxResult.success(voList);
    }

    /**
     * 大屏页面巡护统计，获取当日排班人员数量，当日打卡人员数量，需要distinct
     * 统计排班路径下的所有 应打卡点数量， 当日已打卡点数量
     */
    @PostMapping(value = "/queryRangerStatistics")
    public AjaxResult queryRangerStatistics(@RequestBody(required = false) SchedulesSearcher params) {
        RangerStatisticsVO vo = iSchedulesService.queryRangerStatistics(params);
        return AjaxResult.success(vo);
    }

    /**
     * 查询当天的排班信息，返回排班信息列表，及人员信息列表,不分页
     * 大屏页面
     *
     * @param params
     * @return
     */
    @PostMapping(value = "/queryScheduleByCondition")
    public AjaxResult queryScheduleByCondition(@RequestBody(required = false) SchedulesSearcher params) {
        List<ScheduleVO> voList = iSchedulesService.queryScheduleByCondition(params);
        return AjaxResult.success(voList);
    }

    /**
     * 排班记录详情，带上巡护路径下面的点位信息，标识出已打卡的点位、未打卡的点位
     * 大屏页面
     */
    @GetMapping(value = "/queryScheduleDetail/{scheduleId}")
    public AjaxResult queryScheduleDetail(@PathVariable("scheduleId") Long scheduleId) {
        ScheduleDetailInfoVO vo = iSchedulesService.queryScheduleDetail(scheduleId);
        return AjaxResult.success(vo);
    }

    /**
     * 当日护林员信息统计：当日值班人数，打卡数量，临时任务数量
     * 大屏页面
     *
     * @param regionId
     * @return
     */
    @GetMapping(value = "/queryForesterStatistics")
    public AjaxResult queryForesterStatistics(@RequestParam(required = false) Long regionId) {
        ForesterStatisticsVO statisticsVO = iSchedulesService.queryForesterStatistics(regionId);
        return AjaxResult.success(statisticsVO);
    }

    /**
     * 获取用户当日值班状态
     *
     * @param regionId
     * @param queryDate
     * @return
     */
    @GetMapping(value = "/getUserScheduleStatus")
    public AjaxResult queryScheduleDetail(@RequestParam(required = false) Long regionId, @RequestParam(required = false) Date queryDate) {
        List<UserScheduleStatusVO> userScheduleStatusList = iSchedulesService.getUserScheduleStatus(regionId, queryDate);
        return AjaxResult.success(userScheduleStatusList);
    }

    /**
     * 获取用户当日值班信息
     */
    @GetMapping(value = "/getUserSchedule")
    public AjaxResult getUserSchedule(@RequestParam(required = false) Date date) {
//        Long userId = 102L;
        Long userId = super.getUserId();
        ScheduleVO vo = iSchedulesService.getUserSchedule(userId, date);
        return AjaxResult.success(vo);
    }

    /**
     * 查询月排班及打卡情况
     * 默认查当前月的，如果前端输入了月份，则查指定月份"yyyy-MM"
     */
    @GetMapping(value = "/queryMonthSchedule")
    public AjaxResult queryMonthSchedule(@RequestParam(required = false) YearMonth month) {
//        Long userId = 102L;
        Long userId = super.getUserId();
        List<SchedulesMonthVO> voList = iSchedulesService.queryMonthSchedule(userId, month);
        return AjaxResult.success(voList);
    }

}
