package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.iot.forestfire.domain.Shift;
import com.tcctyn.iot.forestfire.service.IShiftService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 班次表 前端控制器
 * </p>
 *
 * @author liuzh
 * @since 2024-09-24
 */

@RestController
@RequestMapping("/shift")
public class ShiftController extends BaseController {
    @Autowired
    private IShiftService iShiftService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<Shift> pageList = iShiftService.list(current,size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        Shift shift = iShiftService.getById(id);
        return AjaxResult.success(shift);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@Validated @RequestBody Shift params) {
        Integer result = iShiftService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result =  iShiftService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody Shift params) {
        Integer result = iShiftService.update(params);
        return AjaxResult.success(result);
    }

    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(Shift shift) {
        startPage();
        List<Shift> list = iShiftService.selectShiftList(shift);
        return getDataTable(list);
    }

    /**
     * 获取全部班次信息
     * @return
     */
    @GetMapping(value = "/allShift")
    public AjaxResult allShift(){
        return AjaxResult.success(iShiftService.allShift());
    }
}
