package com.tcctyn.iot.forestfire.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.TemporaryTaskVO;
import com.tcctyn.iot.forestfire.domain.TemporaryTask;
import com.tcctyn.iot.forestfire.searcher.BaseIdListSearcher;
import com.tcctyn.iot.forestfire.searcher.temporarytask.TemporaryTaskStatusSearcher;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.ITemporaryTaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 * 临时任务表 前端控制器
 * </p>
 *
 * @author 代超
 * @since 2024-11-01
 */

@RestController
@RequestMapping("/temporaryTask")
public class TemporaryTaskController extends BaseController {


    @Autowired
    private ITemporaryTaskService iTemporaryTaskService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<TemporaryTask> pageList = iTemporaryTaskService.list(current, size);
        return AjaxResult.success(pageList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        TemporaryTask temporaryTask = iTemporaryTaskService.getById(id);
        //做regionName 、 任务人姓名、任务接收人姓名的映射
        TemporaryTaskVO temporaryTaskVO = new TemporaryTaskVO();
        BeanUtils.copyBeanProp(temporaryTask, temporaryTaskVO);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(temporaryTask.getRegionId());
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        temporaryTaskVO.setRegionName(Optional.ofNullable(regionIdToNameMap.get(temporaryTask.getRegionId())).orElse(""));
        temporaryTaskVO.setTaskPersonName(Optional.ofNullable(userIdToNameMap.get(temporaryTask.getTaskPersonId())).orElse(""));
        temporaryTaskVO.setTaskRecipientName(Optional.ofNullable(userIdToNameMap.get(temporaryTask.getTaskRecipientId())).orElse(""));
        return AjaxResult.success(temporaryTaskVO);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@Validated @RequestBody TemporaryTask params) {
        Long userId = super.getUserId();
        params.setIssuerId(userId);
        Integer result = iTemporaryTaskService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iTemporaryTaskService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody TemporaryTask params) {
        Integer result = iTemporaryTaskService.update(params);
        return AjaxResult.success(result);
    }

    /**
     * 临时任务归档
     *
     * @param id
     * @return
     */
    @PostMapping(value = "/archive/{id}")
    public AjaxResult archive(@PathVariable("id") Long id) {
        Integer result = iTemporaryTaskService.archive(id);
        return AjaxResult.success(result);
    }

    /**
     * 临时任务批量归档
     */
    @PostMapping(value = "/archiveBatch")
    public AjaxResult archiveBatch(@RequestBody BaseIdListSearcher<Long> searcher) {
        Integer result = iTemporaryTaskService.archiveBatch(searcher);
        return AjaxResult.success(result);
    }

    /**
     * 临时任务分页查询
     *
     * @param temporaryTask
     * @return
     */
    @GetMapping(value = "/selectList")
    public TableDataInfo selectList(TemporaryTask temporaryTask) {
        startPage();
        List<TemporaryTask> taskList = iTemporaryTaskService.selectList(temporaryTask);
        List<TemporaryTaskVO> voList = BeanUtils.copyList(taskList, TemporaryTaskVO.class);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(item -> {
            item.setRegionName(Optional.ofNullable(regionIdToNameMap.get(item.getRegionId())).orElse(""));
            item.setTaskPersonName(Optional.ofNullable(userIdToNameMap.get(item.getTaskPersonId())).orElse(""));
            item.setTaskRecipientName(Optional.ofNullable(userIdToNameMap.get(item.getTaskRecipientId())).orElse(""));
        });
        return getDataTable(taskList, voList);
    }

    @GetMapping(value = "/selectOneDayList")
    public AjaxResult selectOneDayList(TemporaryTask temporaryTask) {
        List<TemporaryTaskVO> voList = iTemporaryTaskService.selectOneDayList(temporaryTask);
        return AjaxResult.success(voList);
    }


    /**
     * 查询未完成、进行中的临时任务（分页）
     *
     * @param temporaryTask
     * @return
     */
    @GetMapping(value = "/selectUncompletedList")
    public TableDataInfo selectUncompletedList(TemporaryTask temporaryTask) {
        startPage();
        List<TemporaryTask> taskList = iTemporaryTaskService.selectUncompletedList(temporaryTask);
        List<TemporaryTaskVO> voList = BeanUtils.copyList(taskList, TemporaryTaskVO.class);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(item -> {
            item.setRegionName(Optional.ofNullable(regionIdToNameMap.get(item.getRegionId())).orElse(""));
            item.setTaskPersonName(Optional.ofNullable(userIdToNameMap.get(item.getTaskPersonId())).orElse(""));
            item.setTaskRecipientName(Optional.ofNullable(userIdToNameMap.get(item.getTaskRecipientId())).orElse(""));
        });
        return getDataTable(taskList, voList);
    }

    /**
     * 任务接受与完成接口，通过状态字段来判断，任务状态分为未接受、执行中、已完成
     *
     * @param searcher
     * @return
     */
    @PostMapping(value = "/updateTaskStatus")
    public AjaxResult updateTaskStatus(@RequestBody TemporaryTaskStatusSearcher searcher) {
//        Long userId = 102L;
        Long userId = super.getUserId();
        Integer result = iTemporaryTaskService.updateTaskStatus(searcher, userId);
        return AjaxResult.success(result);
    }

    /**
     * 个人未接受临时任务列表分页查询
     */
    @GetMapping(value = "/selectPersonalNotAcceptedTaskList")
    public TableDataInfo selectPersonalNotAcceptedTaskList(TemporaryTask temporaryTask) {
//        Long userId = 100L;
        Long userId = super.getUserId();
        temporaryTask.setTaskPersonId(userId);
        startPage();
        List<TemporaryTask> taskList = iTemporaryTaskService.selectList(temporaryTask);
        List<TemporaryTaskVO> voList = BeanUtils.copyList(taskList, TemporaryTaskVO.class);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(item -> {
            item.setRegionName(Optional.ofNullable(regionIdToNameMap.get(item.getRegionId())).orElse(""));
            item.setTaskPersonName(Optional.ofNullable(userIdToNameMap.get(item.getTaskPersonId())).orElse(""));
            item.setTaskRecipientName(Optional.ofNullable(userIdToNameMap.get(item.getTaskRecipientId())).orElse(""));
        });
        return getDataTable(voList);
    }

    /**
     * 个人进行中、已完成临时任务列表分页查询
     * 个人已经接受了的任务
     */
    @GetMapping(value = "/selectPersonalAcceptedTaskList")
    public TableDataInfo selectPersonalAcceptedTaskList(TemporaryTask temporaryTask) {
//        Long userId = 100L;
        Long userId = super.getUserId();
        temporaryTask.setTaskRecipientId(userId);
        startPage();
        List<TemporaryTask> taskList = iTemporaryTaskService.selectList(temporaryTask);
        List<TemporaryTaskVO> voList = BeanUtils.copyList(taskList, TemporaryTaskVO.class);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(item -> {
            item.setRegionName(Optional.ofNullable(regionIdToNameMap.get(item.getRegionId())).orElse(""));
            item.setTaskPersonName(Optional.ofNullable(userIdToNameMap.get(item.getTaskPersonId())).orElse(""));
            item.setTaskRecipientName(Optional.ofNullable(userIdToNameMap.get(item.getTaskRecipientId())).orElse(""));
        });
        return getDataTable(voList);
    }

    /**
     * APP 端任务下发，根据下发人分页查询任务列表
     */
    @GetMapping(value = "/selectIssuedTaskList")
    public TableDataInfo selectIssuedTaskList(TemporaryTask temporaryTask) {
//        Long userId = 1L;
        Long userId = super.getUserId();
        temporaryTask.setIssuerId(userId);
        startPage();
        List<TemporaryTask> taskList = iTemporaryTaskService.selectList(temporaryTask);
        List<TemporaryTaskVO> voList = BeanUtils.copyList(taskList, TemporaryTaskVO.class);
        Map<Long, String> userIdToNameMap = commonService.userMapIdToName(null);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(item -> {
            item.setRegionName(Optional.ofNullable(regionIdToNameMap.get(item.getRegionId())).orElse(""));
            item.setTaskPersonName(Optional.ofNullable(userIdToNameMap.get(item.getTaskPersonId())).orElse(""));
            item.setTaskRecipientName(Optional.ofNullable(userIdToNameMap.get(item.getTaskRecipientId())).orElse(""));
        });
        return getDataTable(voList);
    }
}
