package com.tcctyn.iot.forestfire.controller;

import com.tcctyn.common.core.web.controller.BaseController;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.web.page.TableDataInfo;
import com.tcctyn.common.core.utils.StringUtils;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.WarehouseSelectVO;
import com.tcctyn.iot.forestfire.VO.WarehouseVO;
import com.tcctyn.iot.forestfire.domain.Warehouse;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IWarehouseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 * 仓库信息 前端控制器
 * </p>
 *
 * @author liuzh
 * @since 2024-10-09
 */

@RestController
@RequestMapping("/warehouse")
public class WarehouseController extends BaseController {


    @Autowired
    private IWarehouseService iWarehouseService;

    @Autowired
    private ICommonService commonService;

    @GetMapping(value = "/list")
    public TableDataInfo list(Warehouse warehouse) {
        startPage();
        List<Warehouse> doList = iWarehouseService.list(warehouse);
        List<WarehouseVO> voList = BeanUtils.copyList(doList, WarehouseVO.class);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(warehouseVO -> {
            String forestName = regionIdToNameMap.get(warehouseVO.getForestId()) == null ? "" : regionIdToNameMap.get(warehouseVO.getForestId());
            String farmName = regionIdToNameMap.get(warehouseVO.getFarmId()) == null ? "" : regionIdToNameMap.get(warehouseVO.getFarmId());
            String fenceName = regionIdToNameMap.get(warehouseVO.getFenceId()) == null ? "" : regionIdToNameMap.get(warehouseVO.getFenceId());

            warehouseVO.setRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
        });
        return getDataTable(doList, voList);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        Warehouse warehouse = iWarehouseService.getById(id);
        return AjaxResult.success(warehouse);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody Warehouse params) {
//        params.setCreatedBy(getUsername());
        Integer result = iWarehouseService.create(params);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iWarehouseService.delete(id);
        return AjaxResult.success(result);
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody Warehouse params) {
//        params.setUpdatedBy(getUsername());
        Integer result = iWarehouseService.update(params);
        return AjaxResult.success(result);
    }

    @GetMapping(value = "/select/{regionId}")
    public AjaxResult selectWarehouse(@PathVariable("regionId") Long regionId) {
        List<WarehouseSelectVO> result = iWarehouseService.selectWarehouse(regionId);
        return AjaxResult.success(result);
    }

    /**
     * 根据前端传入的查询条件查询所有的仓库信息
     *
     * @param warehouse
     * @return
     */
    @GetMapping(value = "/selectAll")
    public AjaxResult selectAll(Warehouse warehouse) {
        List<Warehouse> list = iWarehouseService.selectAll(warehouse);
        //DO 转 VO
        List<WarehouseVO> voList = BeanUtils.copyList(list, WarehouseVO.class);
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        voList.forEach(warehouseVO -> {
            warehouseVO.setRegionName(Optional.ofNullable(regionIdToNameMap.get(warehouseVO.getRegionId())).orElse(""));
        });
        return AjaxResult.success(voList);
    }
}
