package com.tcctyn.iot.forestfire.domain;

import com.baomidou.mybatisplus.annotation.*;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 火情上报表
 * </p>
 *
 * @author 代超
 * @since 2024-11-01
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("fire_report")
public class FireReport implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 火情标题
     */
    @NotBlank(message = "请填写火情上报标题")
    private String fireTitle;

    /**
     * 上报方式：卫星上报、无人机上报、人工上报、监测点（云台/卡口）上报
     */
    private String reportMethod;

    /**
     * 来源火情预警id
     */
    private Long fireWarningId;

    /**
     * 上报设备编号
     */
    private String deviceNo;

    /**
     * 上报设备类型
     */
    private String deviceType;

    /**
     * 上报人员id
     */
    private Long reporterId;

    /**
     * 地址
     */
    @NotBlank(message = "请填写火情地址")
    private String address;

    /**
     * 经度
     */
    private String longitude;

    /**
     * 纬度
     */
    private String latitude;

    /**
     * 高度
     */
    private String height;

    /**
     * 归属区域id
     */
    @NotNull(message = "请选择所属区域")
    private Long regionId;

    /**
     * 林区id
     */
    private Long forestId;

    /**
     * 林场id
     */
    private Long farmId;

    /**
     * 电子围栏id
     */
    private Long fenceId;

    /**
     * 火险等级，有别于火情预警等级
     */
    private String fireHazardLevel;

    /**
     * 起火原因
     */
    private String fireCause;

    /**
     * 失火面积
     */
    private String fireArea;

    /**
     * 预计损失金额
     */
    private String amountLoss;

    /**
     * 火情图片
     */
    private String firePicture;

    /**
     * 火情描述
     */
    private String fireDescription;

    /**
     * 处置状态：未处理、已处理
     */
    private Integer disposalStatus;

    /**
     * 处置说明
     */
    private String disposalInstruction;

    /**
     * 扑救情况
     */
    private String firefightingSituation;

    /**
     * 灾后情况说明
     */
    private String disasterSituation;

    /**
     * 扑救单位
     */
    private String firefightingUnit;

    /**
     * 主要负责人
     */
    private String personInCharge;

    /**
     * 归档状态
     */
    private Integer archiveStatus;

    /**
     * 归档时间
     */
    private Date archiveTime;

    /**
     * 备注信息
     */
    private String remark;

    /**
     * 该条记录是否被删除，1-已删除，0-未删除
     */
    @TableLogic
    private Integer delFlag;

    /**
     * 创建人员姓名
     */
    private String createdBy;

    /**
     * 记录创建时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createdTime;

    /**
     * 修改人员姓名
     */
    private String updatedBy;

    /**
     * 记录修改时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.UPDATE)
    private Date updatedTime;

}
