package com.tcctyn.iot.forestfire.domain;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
/**
 * <p>
 * 请假记录表
 * </p>
 *
 * @author daichao
 * @since 2024-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("leave_record")
public class LeaveRecord implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 请假人id
     */
    private Long userId;

    /**
     * 请假人部门id
     */
    private Long deptId;

    /**
     * 请假类型
     */
    @NotBlank(message = "请选择请假类型")
    private String leaveType;

    /**
     * 开始时间
     */
    @NotNull(message = "请选择开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date startTime;

    /**
     * 结束时间
     */
    @NotNull(message = "请选择结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date endTime;

    /**
     * 请假时长，单位天
     */
    private String leaveDuration;

    /**
     * 请假说明
     */
    @NotBlank(message = "请输入请假说明")
    private String leaveInstruction;

    /**
     * 图片
     */
    private String picture;

    /**
     * 撤销标识：0-未撤销，1-已撤销
     */
    private Integer revokeFlag;

    /**
     * 撤销时间
     */
    private Date revokeTime;

    /**
     * 审批标识：0-未审批，1-已审批
     */
    private Integer approvalFlag;

    /**
     * 审批状态：0-驳回，1-通过
     */
    private Integer approvalStatus;

    /**
     * 驳回说明
     */
    private String rejectionStatement;

    /**
     * 审核人id
     */
    private Long reviewerId;

    /**
     * 审核时间
     */
    private Date reviewTime;

    /**
     * 备注信息
     */
    private String remark;

    /**
     * 该条记录是否被删除，1-已删除，0-未删除
     */
    @TableLogic
    private Integer delFlag;

    /**
     * 创建人员姓名
     */
    private String createdBy;

    /**
     * 记录创建时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createdTime;

    /**
     * 修改人员姓名
     */
    private String updatedBy;

    /**
     * 记录修改时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.UPDATE)
    private Date updatedTime;

}
