package com.tcctyn.iot.forestfire.domain;

import com.baomidou.mybatisplus.annotation.*;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 物资信息
 * </p>
 *
 * @author liuzh
 * @since 2024-10-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("material")
public class Material implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 物资ID
     */
    @TableId(value = "material_id", type = IdType.AUTO)
    private Long materialId;

    /**
     * 仓库id
     */
    @NotNull(message = "仓库不能为空")
    private Long warehouseId;

    /**
     * 编号
     */
    private String serialNumber;

    /**
     * 物资名称
     */
    @NotBlank(message = "物资名称不能为空")
    private String materialName;

    /**
     * 型号
     */
    private String model;

    /**
     * 单位
     */
    private String unit;

    /**
     * 库存
     */
    private BigDecimal stock;

    /**
     * 物资类型
     */
    @NotNull(message = "物资类型不能为空")
    private Integer materialType;

    /**
     * 照片
     */
    private String photo;

    /**
     * 描述
     */
    private String description;

    /**
     * 删除标志
     */
    @TableLogic
    private Integer delFlag;

    /**
     * 创建者
     */
    private String createdBy;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createdTime;

    /**
     * 更新者
     */
    private String updatedBy;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.UPDATE)
    private Date updatedTime;

}
