package com.tcctyn.iot.forestfire.domain;

import com.baomidou.mybatisplus.annotation.*;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 通知公告信息表
 * </p>
 *
 * @author daichao
 * @since 2024-12-17
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("notice_info")
public class NoticeInfo implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 类型：通知、公告
     */
    @NotBlank(message = "请选择通知类型")
    private String type;

    /**
     * 标题
     */
    @NotBlank(message = "请填写标题")
    private String title;

    /**
     * 内容
     */
    @NotBlank(message = "请填写内容")
    private String content;

    /**
     * 文件
     */
    private String file;

    /**
     * 序号
     */
    private Integer sortNo;

    /**
     * 是否启用：0-停用，1-启用
     */
    private Integer enableFlag;

    /**
     * 备注信息
     */
    private String remark;

    /**
     * 该条记录是否被删除，1-已删除，0-未删除
     */
    @TableLogic
    private Integer delFlag;

    /**
     * 创建人员姓名
     */
    private String createdBy;

    /**
     * 记录创建时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createdTime;

    /**
     * 修改人员姓名
     */
    private String updatedBy;

    /**
     * 记录修改时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.UPDATE)
    private Date updatedTime;

}
