package com.tcctyn.iot.forestfire.domain;

import com.baomidou.mybatisplus.annotation.*;
import com.tcctyn.iot.forestfire.BO.PointInfoBO;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 巡护路径表
 * </p>
 *
 * @author 代超
 * @since 2024-09-24
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("patrol_path")
public class PatrolPath implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 路径编号
     */
//    @NotBlank(message = "路径编号不能为空")
    private String pathNo;

    /**
     * 路径名称
     */
    @NotBlank(message = "路径名称不能为空")
    private String pathName;

    /**
     * 归属区域id,取林区、林场、电子围栏中的最下级id
     */
    @NotNull(message = "归属区域id不能为空")
    private Long BelongingRegionId;

    /**
     * 林区id
     */
    private Long forestId;

    /**
     * 林场id
     */
    private Long farmId;

    /**
     * 电子围栏id
     */
    private Long fenceId;

    /**
     * 路径简介
     */
    private String pathIntroduction;


    /**
     * 备注信息
     */
    private String remark;

    /**
     * 该条记录是否被删除，1-已删除，0-未删除
     */
    @TableLogic
    private Integer delFlag;

    /**
     * 创建人员姓名
     */
    private String createdBy;

    /**
     * 记录创建时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createdTime;

    /**
     * 修改人员姓名
     */
    private String updatedBy;

    /**
     * 记录修改时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.UPDATE)
    private Date updatedTime;

    /**
     * 路径坐标信息
     */
    @TableField(exist = false)
    private List<PointInfoBO> pointList;

}
