package com.tcctyn.iot.forestfire.domain;

import com.baomidou.mybatisplus.annotation.*;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 林间道路信息表
 * </p>
 *
 * @author jason
 * @since 2025-03-11
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("road")
public class Road implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 道路编号
     */
    private String roadNo;

    /**
     * 道路名称
     */
    private String roadName;

    /**
     * 道路类型
     */
    private String roadType;

    /**
     * 长度
     */
    private String length;

    /**
     * 宽度
     */
    private String width;

    /**
     * 坐标集合
     */
    private String coordinates;
    /**
     * 归属区域id:取林区、林场、电子围栏的最下级id
     */
    private Long belongingRegionId;

    /**
     * 林区id
     */
    private Long forestId;

    /**
     * 林场id
     */
    private Long farmId;

    /**
     * 电子围栏id
     */
    private Long fenceId;

    /**
     * 备注信息
     */
    private String remark;

    /**
     * 该条记录是否被删除，1-已删除，0-未删除
     */
    private Integer delFlag;

    /**
     * 创建人员姓名
     */
    private String createdBy;

    /**
     * 记录创建时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createdTime;

    /**
     * 修改人员姓名
     */
    private String updatedBy;

    /**
     * 记录修改时间，默认用服务器时间
     */
    @TableField(fill = FieldFill.UPDATE)
    private Date updatedTime;


    /**
     * 道路详细信息
     */
    @TableField(exist = false)
    private List<RoadInfo> roadInfoList;


}
