package com.tcctyn.iot.forestfire.enums;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 设备类型枚举, 用于判断设备类型
 * 暂时与数据字典中的设备类型保持一致，方便做业务上的判断
 * 提供数据字典是方便用户做非业务的设备类型添加
 */
public enum DeviceTypeEnum {
    /**
     * 摄像头
     */
    CAMERA("摄像头"),

    /**
     * 无人机
     */
    DRONE("无人机"),

    /**
     * 机场
     */
    AIRPORT("机场"),

    /**
     * 气象监测仪
     */
    METEOROLOGIC("气象监测仪"),

    /**
     * 红外设备
     */
    INFRARED("红外设备"),

    /**
     * 水泵
     */
    PUMP("水泵"),

    /**
     * 云台摄像头
     */
    CLOUDTERRACE("云台摄像头"),

    /**
     * 卡口摄像头
     */
    BAYONET("卡口摄像头")
    ;

    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    /**
     * 描述
     */
    private String desc;

    DeviceTypeEnum(String desc) {
        this.desc = desc;
    }

    /**
     * 根据枚举名称获取枚举
     *
     * @param enumName
     * @return
     */
    public static DeviceTypeEnum getByEnumName(final String enumName) {
        for (DeviceTypeEnum element : values()) {
            if (element.name().equals(enumName)) {
                return element;
            }
        }
        return CAMERA;
    }

    /**
     * 获取枚举名称和desc所有的映射
     *
     * @return
     */
    public static Map<String, String> getName2DescMap() {

        final Map<String, String> name2TitleMap = new LinkedHashMap<>();

        for (DeviceTypeEnum element : values()) {
            name2TitleMap.put(element.name(), element.desc);
        }

        return name2TitleMap;
    }
}
