package com.tcctyn.iot.forestfire.enums;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 无人机的任务执行状态
 */
public enum DroneTaskStatusEnum {
    /**
     * 未执行
     */
    UNEXECUTED("未执行"),

    /**
     * 执行中
     */
    IN_PROGRESS("执行中"),

    /**
     * 已完成
     */
    COMPLETED("已完成"),

    ;

    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    /**
     * 描述
     */
    private String desc;

    DroneTaskStatusEnum(String desc) {
        this.desc = desc;
    }

    /**
     * 根据枚举名称获取枚举
     *
     * @param enumName
     * @return
     */
    public static DroneTaskStatusEnum getByEnumName(final String enumName) {
        for (DroneTaskStatusEnum element : values()) {
            if (element.name().equals(enumName)) {
                return element;
            }
        }
        return UNEXECUTED;
    }

    /**
     * 获取枚举名称和desc所有的映射
     *
     * @return
     */
    public static Map<String, String> getName2DescMap() {

        final Map<String, String> name2TitleMap = new LinkedHashMap<>();

        for (DroneTaskStatusEnum element : values()) {
            name2TitleMap.put(element.name(), element.desc);
        }

        return name2TitleMap;
    }
}
