package com.tcctyn.iot.forestfire.enums;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 火情预警信息处理状态枚举：
 * 2024-12-02 调整为未处理、已研判、已上报、已归档
 */
public enum FireWarningDisposalStatusEnum {


    /**
     * 未处理
     */
    UNHANDLED("未处理"),

    /**
     * 已研判
     */
    ANALYZED("已研判"),

    /**
     * 已上报
     */
    REPORTED("已上报"),

    /**
     * 已归档
     */
    ARCHIVED("已归档");


    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    /**
     * 描述
     */
    private String desc;

    FireWarningDisposalStatusEnum(String desc) {
        this.desc = desc;
    }

    /**
     * 根据枚举名称获取枚举
     *
     * @param enumName
     * @return
     */
    public static FireWarningDisposalStatusEnum getByEnumName(final String enumName) {
        for (FireWarningDisposalStatusEnum element : values()) {
            if (element.name().equals(enumName)) {
                return element;
            }
        }
        return UNHANDLED;
    }

    /**
     * 获取枚举名称和desc所有的映射
     *
     * @return
     */
    public static Map<String, String> getName2DescMap() {

        final Map<String, String> name2TitleMap = new LinkedHashMap<>();

        for (FireWarningDisposalStatusEnum element : values()) {
            name2TitleMap.put(element.name(), element.desc);
        }

        return name2TitleMap;
    }
}
