package com.tcctyn.iot.forestfire.enums;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 火情上报方式：1、卫星上报、2、无人机上报、3、人工上报、4、监测点上报（云台/卡口）
 */
public enum ReportMethodEnum {

    /**
     * 卫星上报
     */
    SATELLITE_REPORTING("卫星上报"),

    /**
     * 无人机上报
     */
    DRONE_REPORTING("无人机上报"),


    /**
     * 人工上报
     */
    MANUAL_REPORTING("人工上报"),


    /**
     * 监测点上报
     */
    MONITORING_POINT_REPORTING("监测点上报"),

    ;

    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    /**
     * 描述
     */
    private String desc;

    ReportMethodEnum(String desc) {
        this.desc = desc;
    }

    /**
     * 根据枚举名称获取枚举
     *
     * @param enumName
     * @return
     */
    public static ReportMethodEnum getByEnumName(final String enumName) {
        for (ReportMethodEnum element : values()) {
            if (element.name().equals(enumName)) {
                return element;
            }
        }
        return MANUAL_REPORTING;
    }

    /**
     * 获取枚举名称和desc所有的映射
     *
     * @return
     */
    public static Map<String, String> getName2DescMap() {

        final Map<String, String> name2TitleMap = new LinkedHashMap<>();

        for (ReportMethodEnum element : values()) {
            name2TitleMap.put(element.name(), element.desc);
        }

        return name2TitleMap;
    }
}
