package com.tcctyn.iot.forestfire.enums;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 任务状态枚举：未接受、执行中、已完成
 */
public enum TaskStatusEnum {

    /**
     * 未接受
     */
    NOT_ACCEPTED("未接受"),

    /**
     * 执行中
     */
    IN_PROGRESS("执行中"),

    /**
     * 已完成
     */
    COMPLETED("已完成"),

    ;

    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    /**
     * 描述
     */
    private String desc;

    TaskStatusEnum(String desc) {
        this.desc = desc;
    }

    /**
     * 根据枚举名称获取枚举
     *
     * @param enumName
     * @return
     */
    public static TaskStatusEnum getByEnumName(final String enumName) {
        for (TaskStatusEnum element : values()) {
            if (element.name().equals(enumName)) {
                return element;
            }
        }
        return NOT_ACCEPTED;
    }

    /**
     * 获取枚举名称和desc所有的映射
     *
     * @return
     */
    public static Map<String, String> getName2DescMap() {

        final Map<String, String> name2TitleMap = new LinkedHashMap<>();

        for (TaskStatusEnum element : values()) {
            name2TitleMap.put(element.name(), element.desc);
        }

        return name2TitleMap;
    }
}
