package com.tcctyn.iot.forestfire.service;

import com.tcctyn.iot.forestfire.BO.SysUserBO;

import java.util.Map;

/**
 * 放一些通用的接口，方便其他服务调用
 */
public interface ICommonService {

    /**
     * 查询所有的地区及地区名称，方便做映射替换给到前端
     * 支持单个与批量的查询
     * 当regionId为null时，查询所有
     */
    Map<Long, String> regionMapIdToName(Long regionId);

    /**
     * 查询所有的用户名，方便做映射替换给到前端
     * 已经约定，用户的真实姓名存在nick_name 字段中
     * 支持单个与批量的查询
     */
    Map<Long, String> userMapIdToName(Long usrId);

    /**
     * 查询所有的巡护路径，做id跟name的映射
     * 支持单个与批量的查询
     * 当pathId为null时，查询所有
     */
    Map<Long, String> pathMapIdToName(Long pathId);

    /**
     * 查询所有的云台、卡口、水源地归属点位，做id跟name的映射
     * 支持单个与批量的查询
     * 当pointId为null时，查询所有
     *
     * @return
     */
    Map<Long, String> pointMapIdToName(Long pointId);

    /**
     * 查询所有的机场，做机场id跟name的映射
     * 支持单个与批量的查询
     * 当airportId为null时，查询所有
     */
    Map<Long, String> airportMapIdToName(Long airportId);

    /**
     * 查询所有的部门，做部门id跟name的映射
     * 支持单个与批量的查询
     * 当deptId为null时，查询所有
     */
    Map<Long, String> deptMapIdToName(Long deptId);

    /**
     * 查询用户信息
     * 需要的时候可以快捷获取用户信息
     */
    Map<Long, SysUserBO> userMapIdToBO(Long userId);
}
