package com.tcctyn.iot.forestfire.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctyn.iot.forestfire.VO.*;
import com.tcctyn.iot.forestfire.VO.*;
import com.tcctyn.iot.forestfire.domain.FireWarningInfo;
import com.tcctyn.iot.forestfire.searcher.BaseIdListSearcher;
import com.tcctyn.iot.forestfire.searcher.BaseTimeSearcher;
import com.tcctyn.iot.forestfire.searcher.firewarning.FireWarningAnalyzeSearcher;
import com.tcctyn.iot.forestfire.searcher.firewarning.FireWarningConditionSearcher;

import java.util.List;


/**
 * <p>
 * 火情预警信息表 服务类
 * </p>
 *
 * @author 代超
 * @since 2024-10-08
 */
public interface IFireWarningInfoService {

    /**
     * <p>
     * 分页列表查询
     * </p>
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<FireWarningInfo> list(Integer pageNo, Integer pageSize);

    /**
     * <p>
     * 详情接口
     * </p>
     *
     * @param id
     * @return
     */
    FireWarningInfo getById(Long id);

    /**
     * <p>
     * 新增接口
     * </p>
     *
     * @param fireWarningInfo
     * @return
     */
    Integer create(FireWarningInfo fireWarningInfo);


    /**
     * <p>
     * 删除接口
     * </p>
     *
     * @param id
     * @return
     */
    Integer delete(Long id);

    /**
     * <p>
     * 更新接口
     * </p>
     *
     * @param fireWarningInfo
     * @return
     */
    Integer update(FireWarningInfo fireWarningInfo);

    /**
     * 火情预警信息分页查询
     *
     * @param fireWarningInfo
     * @return
     */
    List<FireWarningInfo> selectList(FireWarningInfo fireWarningInfo);

    /**
     * 根据处理状态进行统计
     *
     * @param searcher
     * @return
     */
    FireWarningCountByStatusVO countByDisposalStatus(FireWarningConditionSearcher searcher);

    /**
     * 查询满足条件的所有火情信息
     *
     * @param searcher
     * @return
     */
    List<FireWarningInfoVO> selectByCondition(FireWarningConditionSearcher searcher);

    /**
     * 根据火情等级进行统计
     *
     * @return
     */
    List<FireWarningCountByLevelVO> countByFireLevel(FireWarningConditionSearcher searcher);

    /**
     * 查询最近的十条火情信息
     *
     * @return
     */
    List<FireWarningInfoVO> selectLimit(FireWarningConditionSearcher searcher);

    /**
     * @param searcher
     * @return
     */

    FireWarningStatisticsByDataSourceVO statisticsByDataSource(BaseTimeSearcher searcher);

    /**
     * 火情预警归档
     *
     * @param id
     * @return
     */
    Integer archive(Long id);

    /**
     * 火情预警批量归档
     *
     * @param searcher
     * @return
     */
    Integer archiveBatch(BaseIdListSearcher<Long> searcher);

    /**
     * APP端查询已处理的火情信息（分页）
     *
     * @param fireWarningInfo
     * @return
     */
    List<FireWarningInfo> selectDisposedList(FireWarningInfo fireWarningInfo);

    /**
     * 火情预警信息研判
     *
     * @param searcher
     * @param userId
     * @return
     */
    Integer analyze(FireWarningAnalyzeSearcher searcher, Long userId);

    /**
     * 模拟云台发现火情预警信息，云台为呼马山云台1，设备编号为 BC1733545
     */
    void simulate();

    /**
     * 火情预警信息周边搜索
     * @param id
     * @return
     */
    FireWarningSurroundingVO surroundingSearch(Long id);

    /**
     * 火情预警信息误判接口
     * @param id
     * @return
     */
    Integer misjudge(Long id);
}
