package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.iot.forestfire.domain.NoticeInfo;
import com.tcctyn.iot.forestfire.mapper.NoticeInfoMapper;
import com.tcctyn.iot.forestfire.service.INoticeInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 通知公告信息表 服务实现类
 * </p>
 *
 * @author daichao
 * @since 2024-12-17
 */
@Service
public class NoticeInfoServiceImpl extends ServiceImpl<NoticeInfoMapper, NoticeInfo> implements INoticeInfoService {


    @Resource
    private NoticeInfoMapper noticeInfoMapper;

    @Override
    public Page<NoticeInfo> list(Integer pageNo, Integer pageSize) {
        return noticeInfoMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public NoticeInfo getById(Long id) {
        NoticeInfo noticeInfo = noticeInfoMapper.selectById(id);
        return noticeInfo;
    }

    @Override
    public Integer create(NoticeInfo noticeInfo) {
        return noticeInfoMapper.insert(noticeInfo);
    }

    @Override
    public Integer delete(Long id) {
        return noticeInfoMapper.deleteById(id);
    }

    @Override
    public Integer update(NoticeInfo noticeInfo) {
        return noticeInfoMapper.updateById(noticeInfo);
    }

    @Override
    public List<NoticeInfo> selectList(NoticeInfo noticeInfo) {
        List<NoticeInfo> noticeInfoList = noticeInfoMapper.selectList(new QueryWrapper<NoticeInfo>().lambda()
                .eq(StringUtils.isNotBlank(noticeInfo.getType()), NoticeInfo::getType, noticeInfo.getType())
                .like(StringUtils.isNotBlank(noticeInfo.getTitle()), NoticeInfo::getTitle, noticeInfo.getTitle())
                .eq(Objects.nonNull(noticeInfo.getEnableFlag()), NoticeInfo::getEnableFlag, noticeInfo.getEnableFlag())
                .orderByAsc(NoticeInfo::getSortNo)
                .orderByDesc(NoticeInfo::getEnableFlag)
                .orderByDesc(NoticeInfo::getCreatedTime)
        );
        return noticeInfoList;
    }

}
