package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.BO.PointInfoBO;
import com.tcctyn.iot.forestfire.VO.PatrolPathVO;
import com.tcctyn.iot.forestfire.domain.PatrolPath;
import com.tcctyn.iot.forestfire.domain.PointInfo;
import com.tcctyn.iot.forestfire.mapper.PatrolPathMapper;
import com.tcctyn.iot.forestfire.mapper.PointInfoMapper;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IPatrolPathService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 巡护路径表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-09-24
 */
@Service
public class PatrolPathServiceImpl extends ServiceImpl<PatrolPathMapper, PatrolPath> implements IPatrolPathService {


    @Resource
    private PatrolPathMapper patrolPathMapper;

    @Resource
    private PointInfoMapper pointInfoMapper;

    @Autowired
    private ICommonService commonService;

    @Resource
    private RegionInfoMapper regionInfoMapper;

    @Override
    public Page<PatrolPath> list(Integer pageNo, Integer pageSize) {
        return patrolPathMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public PatrolPathVO getById(Long id) {
        PatrolPath patrolPath = patrolPathMapper.selectById(id);
        //需要查询路径下的点位信息
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().eq("belonging_region_id", id));

        List<PointInfoBO> boList = BeanUtils.copyList(pointInfoList, PointInfoBO.class);
        patrolPath.setPointList(boList);

        PatrolPathVO patrolPathVO = new PatrolPathVO();
        BeanUtils.copyBeanProp(patrolPath, patrolPathVO);
        //组转路径归属名称
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        String forestName = regionIdToNameMap.get(patrolPathVO.getForestId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getForestId());
        String farmName = regionIdToNameMap.get(patrolPathVO.getFarmId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getFarmId());
        String fenceName = regionIdToNameMap.get(patrolPathVO.getFenceId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getFenceId());
        patrolPathVO.setBelongingRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
        return patrolPathVO;
    }

    @Override
    public Integer create(PatrolPath patrolPath) {
        // 巡护路径编号 校验 --2024-10-14 ,路径编号这个字段暂时不做校验，后期看实际需求再处理
//        List<PatrolPath> patrolPaths = patrolPathMapper.selectList(new QueryWrapper<PatrolPath>().eq("path_no", patrolPath.getPathNo()));
//        if (!patrolPaths.isEmpty()) {
//            throw new ServiceException("路径编号已存在");
//        }

        // 巡护路径名称 校验
        List<PatrolPath> patrolPaths2 = patrolPathMapper.selectList(new QueryWrapper<PatrolPath>().eq("path_name", patrolPath.getPathName()));
        if (!patrolPaths2.isEmpty()) {
            throw new ServiceException("路径名称已存在");
        }
        //根据全路径的长度来

        //根据归属区域id来查询全路径id , 然后拆分为 forest_id、farm_id、fence_id
        Long regionId = patrolPath.getBelongingRegionId();
        if (Objects.nonNull(regionId)) {
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    patrolPath.setForestId(regionId);
                    break;
                case 2:
                    patrolPath.setForestId(Long.valueOf(ancestorsArr[1]));
                    patrolPath.setFarmId(regionId);
                    break;
                case 3:
                    patrolPath.setForestId(Long.valueOf(ancestorsArr[1]));
                    patrolPath.setFarmId(Long.valueOf(ancestorsArr[2]));
                    patrolPath.setFenceId(regionId);
                    break;
                case 4:
                    patrolPath.setForestId(Long.valueOf(ancestorsArr[1]));
                    patrolPath.setFarmId(Long.valueOf(ancestorsArr[2]));
                    patrolPath.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }

        Integer result = patrolPathMapper.insert(patrolPath);
        Long patrolPathId = patrolPath.getId();
        // 批量处理点位数据
        if (!patrolPath.getPointList().isEmpty()) {
            //对点位数据做批量插入
            List<PointInfoBO> pointBOList = patrolPath.getPointList();
            List<PointInfo> pointInfoList = new ArrayList<>();
            pointBOList.forEach(bo -> {
                PointInfo pointInfo = new PointInfo();
                pointInfo.setBelongingRegionId(patrolPathId);
                pointInfo.setForestId(patrolPath.getForestId());
                pointInfo.setFarmId(patrolPath.getFarmId());
                pointInfo.setFenceId(patrolPath.getFenceId());
                pointInfo.setLongitude(bo.getLongitude());
                pointInfo.setLatitude(bo.getLatitude());
                pointInfo.setHeight(bo.getHeight());
                pointInfo.setPointType(bo.getPointType());
                pointInfo.setSortNo(bo.getSortNo());
                pointInfo.setCheckInRadius(bo.getCheckInRadius());
                pointInfo.setDelFlag(0);
                pointInfoList.add(pointInfo);
            });
            //BO 需要转换为 DO 才能插入
            pointInfoMapper.insertBatchSomeColumn(pointInfoList);
        }

        return result;
    }

    @Override
    public Integer delete(Long id) {
        Integer result = patrolPathMapper.deleteById(id);
        //删除巡护路径时，需要将关联的点位信息删除
        pointInfoMapper.delete(new QueryWrapper<PointInfo>().eq("belonging_region_id", id));
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer update(PatrolPath patrolPath) {
        // 巡护路径编号 校验
//        List<PatrolPath> patrolPaths = patrolPathMapper.selectList(new QueryWrapper<PatrolPath>().eq("path_no", patrolPath.getPathNo()).ne("id", patrolPath.getId()));
//        if (!patrolPaths.isEmpty()) {
//            throw new ServiceException("路径编号已存在");
//        }

        // 巡护路径名称 校验
        List<PatrolPath> patrolPaths2 = patrolPathMapper.selectList(new QueryWrapper<PatrolPath>().eq("path_name", patrolPath.getPathName()).ne("id", patrolPath.getId()));
        if (!patrolPaths2.isEmpty()) {
            throw new ServiceException("路径名称已存在");
        }

        //如果巡护路径修改涉及到归属区域的修改，需要调整相应的forest_id、farm_id、fence_id
        PatrolPath originalPatrolPath = patrolPathMapper.selectById(patrolPath.getId());
        if (!Objects.equals(originalPatrolPath.getBelongingRegionId(), patrolPath.getBelongingRegionId())) {
            Long regionId = patrolPath.getBelongingRegionId();
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    patrolPath.setForestId(regionId);
                    break;
                case 2:
                    patrolPath.setForestId(Long.valueOf(ancestorsArr[1]));
                    patrolPath.setFarmId(regionId);
                    break;
                case 3:
                    patrolPath.setForestId(Long.valueOf(ancestorsArr[1]));
                    patrolPath.setFarmId(Long.valueOf(ancestorsArr[2]));
                    patrolPath.setFenceId(regionId);
                    break;
                case 4:
                    patrolPath.setForestId(Long.valueOf(ancestorsArr[1]));
                    patrolPath.setFarmId(Long.valueOf(ancestorsArr[2]));
                    patrolPath.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }

        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().eq("belonging_region_id", patrolPath.getId()));
        List<Long> originalPointIdList;
        List<Long> existsPointIdList = new ArrayList<>();
        if (!pointInfoList.isEmpty()) {
            originalPointIdList = pointInfoList.stream().map(PointInfo::getId).collect(Collectors.toList());
        } else {
            originalPointIdList = new ArrayList<>();
        }

        //对点位信息做更新、新增区分
        if (!patrolPath.getPointList().isEmpty()) {
            //对点位数据做批量插入
            List<PointInfoBO> pointBOList = patrolPath.getPointList();
            //新增数据集合
            List<PointInfo> addList = new ArrayList<>();
            //更新数据集合
            List<PointInfo> updateList = new ArrayList<>();
            //删除数据集合  需要将原先不包含的点位信息删除
            List<Long> deleteList = new ArrayList<>();

            for (PointInfoBO bo : pointBOList) {
                PointInfo pointInfo = new PointInfo();

                if (Objects.isNull(bo.getId())) {
                    pointInfo.setBelongingRegionId(patrolPath.getId());
                    pointInfo.setForestId(patrolPath.getForestId());
                    pointInfo.setFarmId(patrolPath.getFarmId());
                    pointInfo.setFenceId(patrolPath.getFenceId());
                    pointInfo.setLongitude(bo.getLongitude());
                    pointInfo.setLatitude(bo.getLatitude());
                    pointInfo.setHeight(bo.getHeight());
                    pointInfo.setPointType(bo.getPointType());
                    pointInfo.setSortNo(bo.getSortNo());
                    pointInfo.setCheckInRadius(bo.getCheckInRadius());
                    pointInfo.setDelFlag(0);
                    addList.add(pointInfo);
                } else {
                    pointInfo.setId(bo.getId());
                    pointInfo.setLongitude(bo.getLongitude());
                    pointInfo.setLatitude(bo.getLatitude());
                    pointInfo.setHeight(bo.getHeight());
                    pointInfo.setPointType(bo.getPointType());
                    pointInfo.setSortNo(bo.getSortNo());
                    pointInfo.setCheckInRadius(bo.getCheckInRadius());
                    updateList.add(pointInfo);
                }
                existsPointIdList.add(bo.getId());
            }
            if (!addList.isEmpty()) {
                pointInfoMapper.insertBatchSomeColumn(addList);
            }
            if (!updateList.isEmpty()) {
                pointInfoMapper.updateBatch(updateList);
            }
            //过滤出不在现在集合中的点位id
            deleteList = originalPointIdList.stream().filter(pointId -> !existsPointIdList.contains(pointId)).collect(Collectors.toList());
            if (!deleteList.isEmpty()) {
                pointInfoMapper.deleteBatchIds(deleteList);
            }
        }
        Integer result = patrolPathMapper.updateById(patrolPath);
        return result;
    }

    @Override
    public List<PatrolPath> selectList(PatrolPath patrolPath) {
        List<PatrolPath> patrolPathList = patrolPathMapper.selectList(new QueryWrapper<PatrolPath>()
                .like(StringUtils.isNotBlank(patrolPath.getPathName()), "path_name", patrolPath.getPathName())
                .eq(Objects.nonNull(patrolPath.getBelongingRegionId()), "belonging_region_id", patrolPath.getBelongingRegionId())
                .eq(Objects.nonNull(patrolPath.getForestId()), "forest_id", patrolPath.getForestId())
                .eq(Objects.nonNull(patrolPath.getFarmId()), "farm_id", patrolPath.getFarmId())
                .eq(Objects.nonNull(patrolPath.getFenceId()), "fence_id", patrolPath.getFenceId())
                .orderByAsc("created_time"));
        return patrolPathList;
    }

    @Override
    public List<PatrolPath> selectByRegionId(Long regionId) {
        return patrolPathMapper.selectList(new QueryWrapper<PatrolPath>().eq("belonging_region_id", regionId));
    }

    @Override
    public List<PatrolPathVO> queryByRegion(PatrolPath patrolPath) {
        List<PatrolPath> patrolPathList = patrolPathMapper.selectList(new QueryWrapper<PatrolPath>().lambda()
                .eq(Objects.nonNull(patrolPath.getForestId()), PatrolPath::getForestId, patrolPath.getForestId())
                .eq(Objects.nonNull(patrolPath.getFarmId()), PatrolPath::getFarmId, patrolPath.getFarmId())
                .eq(Objects.nonNull(patrolPath.getFenceId()), PatrolPath::getFenceId, patrolPath.getFenceId())
        );
        List<Long> patrolPathIdList = patrolPathList.stream().map(PatrolPath::getId).collect(Collectors.toList());
        //查询路径下面的点位信息
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda().in(!CollectionUtils.isEmpty(patrolPathIdList), PointInfo::getBelongingRegionId, patrolPathIdList));
        if (!pointInfoList.isEmpty()) {
            Map<Long, List<PointInfo>> pointInfoMap = pointInfoList.stream().collect(Collectors.groupingBy(PointInfo::getBelongingRegionId));
            for (PatrolPath item : patrolPathList) {
                item.setPointList(BeanUtils.copyList(pointInfoMap.get(item.getId()), PointInfoBO.class));
            }
        }
        //DO -> VO
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        List<PatrolPathVO> voList = BeanUtils.copyList(patrolPathList, PatrolPathVO.class);
        voList.forEach(patrolPathVO -> {
            String forestName = regionIdToNameMap.get(patrolPathVO.getForestId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getForestId());
            String farmName = regionIdToNameMap.get(patrolPathVO.getFarmId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getFarmId());
            String fenceName = regionIdToNameMap.get(patrolPathVO.getFenceId()) == null ? "" : regionIdToNameMap.get(patrolPathVO.getFenceId());
            patrolPathVO.setBelongingRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
        });

        return voList;
    }

}
