package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.DeviceInfoSimpleVO;
import com.tcctyn.iot.forestfire.VO.PointAndDeviceInfoVO;
import com.tcctyn.iot.forestfire.VO.PointInfoVO;
import com.tcctyn.iot.forestfire.VO.PointStatisticsByPointTypeVO;
import com.tcctyn.iot.forestfire.domain.DeviceInfo;
import com.tcctyn.iot.forestfire.domain.PointInfo;
import com.tcctyn.iot.forestfire.enums.PointTypeEnum;
import com.tcctyn.iot.forestfire.mapper.DeviceInfoMapper;
import com.tcctyn.iot.forestfire.mapper.PointInfoMapper;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IPointInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 点位信息表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-09-24
 */
@Service
public class PointInfoServiceImpl extends ServiceImpl<PointInfoMapper, PointInfo> implements IPointInfoService {


    @Resource
    private PointInfoMapper pointInfoMapper;

    @Autowired
    private ICommonService commonService;

    @Resource
    private RegionInfoMapper regionInfoMapper;

    @Resource
    private DeviceInfoMapper deviceInfoMapper;

    @Override
    public Page<PointInfo> list(Integer pageNo, Integer pageSize) {
        return pointInfoMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public PointInfoVO getById(Long id) {
        PointInfo pointInfo = pointInfoMapper.selectById(id);
        PointInfoVO pointInfoVO = new PointInfoVO();
        BeanUtils.copyProperties(pointInfo, pointInfoVO);
        //拼接归属区域名称：
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        String forestName = regionIdToNameMap.get(pointInfoVO.getForestId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getForestId());
        String farmName = regionIdToNameMap.get(pointInfoVO.getFarmId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getFarmId());
        String fenceName = regionIdToNameMap.get(pointInfoVO.getFenceId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getFenceId());
        pointInfoVO.setBelongingRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
        return pointInfoVO;
    }

    @Override
    public Integer create(PointInfo pointInfo) {
        //编号暂时不做校验
//            List<PointInfo> pointInfoList1 = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().eq("point_no", pointInfo.getPointNo()));
//            if (!pointInfoList1.isEmpty()) {
//                throw new ServiceException(PointTypeEnum.getByEnumName(pointInfo.getPointType()).getDesc() + "编号已存在");
//            }
        List<PointInfo> pointInfoList2 = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().eq("point_name", pointInfo.getPointName()));
        if (!pointInfoList2.isEmpty()) {
            throw new ServiceException(PointTypeEnum.getByEnumName(pointInfo.getPointType()).getDesc() + "名称已存在");
        }

        Long regionId = pointInfo.getBelongingRegionId();
        if (Objects.nonNull(regionId)) {
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    pointInfo.setForestId(regionId);
                    break;
                case 2:
                    pointInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    pointInfo.setFarmId(regionId);
                    break;
                case 3:
                    pointInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    pointInfo.setFarmId(Long.valueOf(ancestorsArr[2]));
                    pointInfo.setFenceId(regionId);
                    break;
                case 4:
                    pointInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    pointInfo.setFarmId(Long.valueOf(ancestorsArr[2]));
                    pointInfo.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }

        return pointInfoMapper.insert(pointInfo);
    }

    @Override
    public Integer delete(Long id) {
        //TODO 删除点位信息时，需要提醒用户该点位信息下是否有设备，如果有设备，则不允许删除
        return pointInfoMapper.deleteById(id);
    }

    @Override
    public Integer update(PointInfo pointInfo) {

//            List<PointInfo> pointInfoList1 = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().eq("point_no", pointInfo.getPointNo()).ne("id", pointInfo.getId()));
//            if (!pointInfoList1.isEmpty()) {
//                throw new ServiceException(PointTypeEnum.getByEnumName(pointInfo.getPointType()).getDesc() + "编号已存在");
//            }
        List<PointInfo> pointInfoList2 = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().eq("point_name", pointInfo.getPointName()).ne("id", pointInfo.getId()));
        if (!pointInfoList2.isEmpty()) {
            throw new ServiceException(PointTypeEnum.getByEnumName(pointInfo.getPointType()).getDesc() + "名称已存在");
        }

        PointInfo originalPointInfo = pointInfoMapper.selectById(pointInfo.getId());
        if (!Objects.equals(originalPointInfo.getBelongingRegionId(), pointInfo.getBelongingRegionId())) {
            Long regionId = pointInfo.getBelongingRegionId();
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    pointInfo.setForestId(regionId);
                    break;
                case 2:
                    pointInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    pointInfo.setFarmId(regionId);
                    break;
                case 3:
                    pointInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    pointInfo.setFarmId(Long.valueOf(ancestorsArr[2]));
                    pointInfo.setFenceId(regionId);
                    break;
                case 4:
                    pointInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    pointInfo.setFarmId(Long.valueOf(ancestorsArr[2]));
                    pointInfo.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }

        return pointInfoMapper.updateById(pointInfo);
    }

    @Override
    public List<PointInfo> selectAll(PointInfo pointInfo) {
        //接口调整，可灵活的的根据不同的参数来查询，方便查询云台、卡口、机场
        return pointInfoMapper.selectList(new QueryWrapper<PointInfo>()
                .like(StringUtils.isNotBlank(pointInfo.getPointNo()), "point_no", pointInfo.getPointNo())
                .like(StringUtils.isNotBlank(pointInfo.getPointName()), "point_name", pointInfo.getPointName())
                .eq(StringUtils.isNotBlank(pointInfo.getPointType()), "point_type", pointInfo.getPointType())
                .eq(Objects.nonNull(pointInfo.getForestId()), "forest_id", pointInfo.getForestId())
                .eq(Objects.nonNull(pointInfo.getFarmId()), "farm_id", pointInfo.getFarmId())
                .eq(Objects.nonNull(pointInfo.getFenceId()), "fence_id", pointInfo.getFenceId())
                .in(!CollectionUtils.isEmpty(pointInfo.getPointTypeList()), "point_type", pointInfo.getPointTypeList())
        );
    }

    @Override
    public Integer deleteBatch(List<Long> idList) {
        return pointInfoMapper.deleteBatchIds(idList);
    }

    @Override
    public List<PointInfo> selectList(PointInfo pointInfo) {
        // 此处pointType 是必填的
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                .eq(StringUtils.isNotBlank(pointInfo.getPointType()), PointInfo::getPointType, pointInfo.getPointType())
                .in(StringUtils.isBlank(pointInfo.getPointType()), PointInfo::getPointType, Arrays.asList(PointTypeEnum.PTZ.name(), PointTypeEnum.BAYONET.name(), PointTypeEnum.WATER_SOURCE.name()))
                .eq(Objects.nonNull(pointInfo.getBelongingRegionId()), PointInfo::getBelongingRegionId, pointInfo.getBelongingRegionId())
                .eq(Objects.nonNull(pointInfo.getForestId()), PointInfo::getForestId, pointInfo.getForestId())
                .eq(Objects.nonNull(pointInfo.getFarmId()), PointInfo::getFarmId, pointInfo.getFarmId())
                .eq(Objects.nonNull(pointInfo.getFenceId()), PointInfo::getFenceId, pointInfo.getFenceId())
                .like(StringUtils.isNotBlank(pointInfo.getAdministrativeDivision()), PointInfo::getAdministrativeDivision, pointInfo.getAdministrativeDivision())
                .like(StringUtils.isNotBlank(pointInfo.getPointName()), PointInfo::getPointName, pointInfo.getPointName())
                .orderByDesc(PointInfo::getCreatedTime)
        );
        return pointInfoList;
    }

    @Override
    public List<PointAndDeviceInfoVO> queryMonitoringPoint(PointInfo pointInfo) {
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                .eq(StringUtils.isNotBlank(pointInfo.getPointType()), PointInfo::getPointType, pointInfo.getPointType())
                .in(StringUtils.isBlank(pointInfo.getPointType()), PointInfo::getPointType, Arrays.asList(PointTypeEnum.PTZ.name(), PointTypeEnum.BAYONET.name()))
                .eq(Objects.nonNull(pointInfo.getForestId()), PointInfo::getForestId, pointInfo.getForestId())
                .eq(Objects.nonNull(pointInfo.getFarmId()), PointInfo::getFarmId, pointInfo.getFarmId())
                .eq(Objects.nonNull(pointInfo.getFenceId()), PointInfo::getFenceId, pointInfo.getFenceId())
        );
        //DO -> VO ， 将所属区域名称设置进去
        Map<Long, String> regionIdToNameMap = commonService.regionMapIdToName(null);
        List<PointAndDeviceInfoVO> voList = BeanUtils.copyList(pointInfoList, PointAndDeviceInfoVO.class);
        List<Long> pointIdList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(voList)) {
            pointIdList = voList.stream().map(PointAndDeviceInfoVO::getId).collect(Collectors.toList());
            //需要将设备信息挂在云台、卡口下面
            List<DeviceInfo> deviceInfoList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                    .in(DeviceInfo::getPointId, pointIdList)
                    .eq(DeviceInfo::getEnableFlag, 1)
            );
            List<DeviceInfoSimpleVO> simpleVOList = BeanUtils.copyList(deviceInfoList, DeviceInfoSimpleVO.class);
            Map<Long, List<DeviceInfoSimpleVO>> deviceInfoMap = simpleVOList.stream().collect(Collectors.groupingBy(DeviceInfoSimpleVO::getPointId));
            voList.forEach(pointInfoVO -> {
                String forestName = regionIdToNameMap.get(pointInfoVO.getForestId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getForestId());
                String farmName = regionIdToNameMap.get(pointInfoVO.getFarmId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getFarmId());
                String fenceName = regionIdToNameMap.get(pointInfoVO.getFenceId()) == null ? "" : regionIdToNameMap.get(pointInfoVO.getFenceId());
                pointInfoVO.setBelongingRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
                pointInfoVO.setDeviceInfoList(deviceInfoMap.get(pointInfoVO.getId()) == null ? new ArrayList<>() : deviceInfoMap.get(pointInfoVO.getId()));
            });
        }
        return voList;
    }
/**
 * 设备快捷修改启用/停用状态接口
 */
    @Override
    public Integer updateEnableFlag(PointInfo pointInfo) {
        Integer result = pointInfoMapper.updateById(pointInfo);
        return result;
    }//sgq2025.3.20

    @Override
    public PointStatisticsByPointTypeVO statisticsByPointType() {
        PointStatisticsByPointTypeVO vo = pointInfoMapper.statisticsByPointType();
        vo.setTotalNum(vo.getPtzNum() + vo.getBayonetNum() + vo.getWaterSourceNum());
        return vo;
    }

}
