package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.web.domain.model.LoginUser;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.StringUtils;
import com.tcctyn.iot.forestfire.domain.Road;
import com.tcctyn.iot.forestfire.domain.RoadInfo;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.mapper.RoadInfoMapper;
import com.tcctyn.iot.forestfire.mapper.RoadMapper;
import com.tcctyn.iot.forestfire.searcher.road.RoadSearcher;
import com.tcctyn.iot.forestfire.service.IRoadInfoService;
import com.tcctyn.iot.forestfire.service.IRoadService;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static com.tcctyn.common.core.utils.SecurityUtils.getLoginUser;

/**
 * <p>
 * 林间道路信息表 服务实现类
 * </p>
 *
 * @author jason
 * @since 2025-03-11
 */
@Service
public class RoadServiceImpl extends ServiceImpl<RoadMapper, Road> implements IRoadService {
    @Resource
    private RoadMapper roadMapper;

    @Resource
    private RegionInfoMapper regionInfoMapper;
    @Resource
    private RoadInfoMapper roadInfoMapper;

    @Resource
    private IRoadInfoService roadInfoService;

    @Override
    public Integer create(Road road) {
        List<Road> rList = roadMapper.selectList(new QueryWrapper<Road>().eq("road_name", road.getRoadName()));
        if (!rList.isEmpty()) {
            throw new ServiceException("道路名称已存在");
        }

        Long regionId = road.getBelongingRegionId();
        if (Objects.nonNull(regionId)) {
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    road.setForestId(regionId);
                    break;
                case 2:
                    road.setForestId(Long.valueOf(ancestorsArr[1]));
                    road.setFarmId(regionId);
                    break;
                case 3:
                    road.setForestId(Long.valueOf(ancestorsArr[1]));
                    road.setFarmId(Long.valueOf(ancestorsArr[2]));
                    road.setFenceId(regionId);
                    break;
                case 4:
                    road.setForestId(Long.valueOf(ancestorsArr[1]));
                    road.setFarmId(Long.valueOf(ancestorsArr[2]));
                    road.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }
        LoginUser loginUser = getLoginUser();
        road.setCreatedBy(loginUser.getUsername());
        Integer result = roadMapper.insert(road);

        // 批量处理特征点数据
        if (ObjectUtils.isNotEmpty(road.getRoadInfoList())) {
            //道路特征数据
            List<RoadInfo> infoList = road.getRoadInfoList();
            List<RoadInfo> roadInfoList = new ArrayList<>();
            infoList.forEach(info -> {
                RoadInfo roadInfo = new RoadInfo();
                roadInfo.setRoadId(road.getId());
                roadInfo.setAddress(info.getAddress());
                roadInfo.setCoordinates(info.getCoordinates());
                roadInfo.setSortNo(info.getSortNo());
                roadInfo.setDelFlag(0);
                roadInfo.setSlope(info.getSlope());
                roadInfo.setCamber(info.getCamber());
                roadInfo.setWidth(info.getWidth());
                roadInfo.setRoadPicture(info.getRoadPicture());
                roadInfo.setRemark(info.getRemark());
                roadInfo.setFeatureType(info.getFeatureType());
                roadInfoList.add(roadInfo);
            });
            roadInfoService.saveOrUpdateBatch(roadInfoList);
        }


        return result;
    }

    @Override
    public Integer update(Road road) {
        List<Road> rList = roadMapper.selectList(new QueryWrapper<Road>().eq("road_name", road.getRoadName()).ne("id", road.getId()));
        if (!rList.isEmpty()) {
            throw new ServiceException("道路名称已存在");
        }

        Road roadV = roadMapper.selectById(road.getId());
        if(roadV==null){
            throw new ServiceException("数据不存在");
        }
        if (!Objects.equals(roadV.getBelongingRegionId(), road.getBelongingRegionId())) {
            Long regionId = road.getBelongingRegionId();
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    road.setForestId(regionId);
                    break;
                case 2:
                    road.setForestId(Long.valueOf(ancestorsArr[1]));
                    road.setFarmId(regionId);
                    break;
                case 3:
                    road.setForestId(Long.valueOf(ancestorsArr[1]));
                    road.setFarmId(Long.valueOf(ancestorsArr[2]));
                    road.setFenceId(regionId);
                    break;
                case 4:
                    road.setForestId(Long.valueOf(ancestorsArr[1]));
                    road.setFarmId(Long.valueOf(ancestorsArr[2]));
                    road.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }

        if (!road.getRoadInfoList().isEmpty()) {
            List<RoadInfo> infoList = road.getRoadInfoList();

            //新增和修改的数组
            List<RoadInfo> upList = new ArrayList<>();

            for (RoadInfo ri : infoList) {
                RoadInfo info = new RoadInfo();
                if (Objects.isNull(ri.getId())) {
                    info.setRoadId(road.getId());
                    info.setAddress(ri.getAddress());
                    info.setCoordinates(ri.getCoordinates());
                    info.setWidth(ri.getWidth());
                    info.setFeatureType(ri.getFeatureType());
                    info.setSlope(ri.getSlope());
                    info.setCamber(ri.getCamber());
                    info.setRoadPicture(ri.getRoadPicture());
                    info.setRemark(ri.getRemark());
                    info.setSortNo(ri.getSortNo());
                    info.setDelFlag(0);
                    LoginUser loginUser = getLoginUser();
                    info.setCreatedBy(loginUser.getUsername());
                    upList.add(info);
                } else {
                    LoginUser loginUser = getLoginUser();
                    ri.setUpdatedBy(loginUser.getUsername());
                    upList.add(ri);
                }
            }
            roadInfoService.saveOrUpdateBatch(upList);
        }
        Integer result = roadMapper.updateById(road);
        return result;
    }

    @Override
    public Page<Road> selectPageList(RoadSearcher roadSearcher) {
        QueryWrapper<Road> wrapper = new QueryWrapper<Road>().eq("del_flag", 0);
        // 动态添加条件
        if (StringUtils.isNotEmpty(roadSearcher.getRoadNo())) {
            wrapper.like("road_no", roadSearcher.getRoadNo());
        }
        if (StringUtils.isNotEmpty(roadSearcher.getRoadName())) {
            wrapper.like("road_name", roadSearcher.getRoadName());
        }
        if (roadSearcher.getBelongingRegionId() != null) {
            wrapper.eq("belonging_region_id", roadSearcher.getBelongingRegionId());
        }
        if (roadSearcher.getForestId() != null) {
            wrapper.eq("forest_id", roadSearcher.getForestId());
        }
        if (roadSearcher.getFarmId() != null) {
            wrapper.eq("farm_id", roadSearcher.getFarmId());
        }
        if (roadSearcher.getFenceId() != null) {
            wrapper.eq("fence_id", roadSearcher.getFenceId());
        }
        return roadMapper.selectPage(new Page<>(roadSearcher.getPageNo(), roadSearcher.getPageSize()), wrapper);
    }
}
