package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.utils.DateUtils;
import com.tcctyn.iot.forestfire.domain.SchedulePlan;
import com.tcctyn.iot.forestfire.domain.ScheduleTemplate;
import com.tcctyn.iot.forestfire.domain.Schedules;
import com.tcctyn.iot.forestfire.mapper.SchedulePlanMapper;
import com.tcctyn.iot.forestfire.service.IRegionInfoService;
import com.tcctyn.iot.forestfire.service.ISchedulePlanService;
import com.tcctyn.iot.forestfire.service.IScheduleTemplateService;
import com.tcctyn.iot.forestfire.service.ISchedulesService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
* <p>
* 排班计划 服务实现类
* </p>
*
* @author liuzh
* @since 2024-10-16
*/
@Service
public class SchedulePlanServiceImpl extends ServiceImpl<SchedulePlanMapper,SchedulePlan> implements ISchedulePlanService {

    @Resource
    private SchedulePlanMapper schedulePlanMapper;

    @Resource
    private ISchedulesService schedulesService;

    @Resource
    private IScheduleTemplateService scheduleTemplateService;

    @Resource
    private IRegionInfoService regionInfoService;

    @Override
    public List<SchedulePlan> list(SchedulePlan schedulePlan) {
        List<SchedulePlan> schedulePlanList = schedulePlanMapper.selectList(new QueryWrapper<SchedulePlan>().setEntity(schedulePlan));
        return schedulePlanList;
    }

    @Override
    public SchedulePlan getById(Long id) {
        SchedulePlan schedulePlan = schedulePlanMapper.selectById(id);
        return schedulePlan;
    }

    @Override
    public Integer create(SchedulePlan schedulePlan) {
        //根据 belongingRegionId 查询出所属区域信息
        RegionInfo regionInfo = regionInfoService.getById(schedulePlan.getBelongingRegionId());
        String[] splitAncestors = regionInfo.getAncestors().split(",");
        if(splitAncestors.length == 2){
            schedulePlan.setForestId(Long.valueOf(splitAncestors[1]));
            schedulePlan.setFarmId(regionInfo.getId());
        } else if (splitAncestors.length == 3) {
            schedulePlan.setForestId(Long.valueOf(splitAncestors[1]));
            schedulePlan.setFarmId(Long.valueOf(splitAncestors[2]));
            schedulePlan.setFenceId(regionInfo.getId());
        } else {
            schedulePlan.setForestId(regionInfo.getId());
        }
        return schedulePlanMapper.insert(schedulePlan);
    }

    @Override
    public Integer delete(Long id) {
        return schedulePlanMapper.deleteById(id);
    }

    @Override
    public Integer update(SchedulePlan schedulePlan) {
        //检查belongRegionId是否被修改
        if (!schedulePlan.getBelongingRegionId().equals(schedulePlanMapper.selectById(schedulePlan.getPlanId()).getBelongingRegionId())){
            //belongingRegionId被修改，需要更新表中的 forestId, farmId, fenceId
            RegionInfo regionInfo = regionInfoService.getById(schedulePlan.getBelongingRegionId());
            String[] splitAncestors = regionInfo.getAncestors().split(",");
            if(splitAncestors.length == 2){
                schedulePlan.setForestId(Long.valueOf(splitAncestors[1]));
                schedulePlan.setFarmId(regionInfo.getId());
            } else if (splitAncestors.length == 3) {
                schedulePlan.setForestId(Long.valueOf(splitAncestors[1]));
                schedulePlan.setFarmId(Long.valueOf(splitAncestors[2]));
                schedulePlan.setFenceId(regionInfo.getId());
            } else {
                schedulePlan.setForestId(regionInfo.getId());
            }
        }
        return schedulePlanMapper.updateById(schedulePlan);
    }

    /**
     * 生成排班信息
     * @param schedulePlan
     * @return
     */
    @Override
    public Integer generate(SchedulePlan schedulePlan) {
        int result = 0;
        Date startDate = schedulePlan.getStartDate();
        Date endDate = schedulePlan.getEndDate();
        //如果传入开始日期在当前日期之前，用当前日期替换
        if (schedulePlan.getStartDate().before(DateUtils.getNowDate())){
            startDate = DateUtils.getNowDate();
        }

        List<ScheduleTemplate> scheduleTemplates = scheduleTemplateService.getByPlanId(schedulePlan.getPlanId());
        if (scheduleTemplates.size() == 0){
            return 0;
        }
        //遍历scheduleTemplates，添加排班信息
        for (ScheduleTemplate scheduleTemplate : scheduleTemplates){
            //从开始日期到结束日期循环添加排班信息
            List<Schedules> newList = new ArrayList<>();
            for (Date date = startDate; date.before(DateUtils.addDays(endDate, 1)); date = DateUtils.addDays(date,1)){
                //获取当前日期对应的星期几
                int dayOfWeek = DateUtils.getWeek(date);
                switch (dayOfWeek) {
                    case 1:
                        //周一
                        //判断用户ID列表是否为空
                        if(!scheduleTemplate.getMonUsers().isEmpty()){
                            for (String userId : scheduleTemplate.getMonUsers().split(",")) {
                                Schedules schedules = new Schedules();
                                schedules.setPlanId(scheduleTemplate.getPlanId());
                                schedules.setUserId(Long.valueOf(userId));
                                schedules.setBelongingRegionId(scheduleTemplate.getBelongingRegionId());
                                schedules.setForestId(scheduleTemplate.getForestId());
                                schedules.setFarmId(scheduleTemplate.getFarmId());
                                schedules.setFenceId(scheduleTemplate.getFenceId());
                                schedules.setPathId(scheduleTemplate.getPathId());
                                schedules.setDayOfWeek(1);
                                schedules.setDateOfWeek(date);
                                newList.add(schedules);
                            }
                        }
                        break;
                    case 2:
                        //周二
                        if (!scheduleTemplate.getTueUsers().isEmpty()){
                            for (String userId : scheduleTemplate.getTueUsers().split(",")) {
                                Schedules schedules = new Schedules();
                                schedules.setPlanId(scheduleTemplate.getPlanId());
                                schedules.setUserId(Long.valueOf(userId));
                                schedules.setBelongingRegionId(scheduleTemplate.getBelongingRegionId());
                                schedules.setForestId(scheduleTemplate.getForestId());
                                schedules.setFarmId(scheduleTemplate.getFarmId());
                                schedules.setFenceId(scheduleTemplate.getFenceId());
                                schedules.setPathId(scheduleTemplate.getPathId());
                                schedules.setDayOfWeek(2);
                                schedules.setDateOfWeek(date);
                                newList.add(schedules);
                            }
                        }
                        break;
                    case 3:
                        //周三
                        if (!scheduleTemplate.getWedUsers().isEmpty()){
                            for (String userId : scheduleTemplate.getWedUsers().split(",")) {
                                Schedules schedules = new Schedules();
                                schedules.setPlanId(scheduleTemplate.getPlanId());
                                schedules.setUserId(Long.valueOf(userId));
                                schedules.setBelongingRegionId(scheduleTemplate.getBelongingRegionId());
                                schedules.setForestId(scheduleTemplate.getForestId());
                                schedules.setFarmId(scheduleTemplate.getFarmId());
                                schedules.setFenceId(scheduleTemplate.getFenceId());
                                schedules.setPathId(scheduleTemplate.getPathId());
                                schedules.setDayOfWeek(3);
                                schedules.setDateOfWeek(date);
                                newList.add(schedules);
                            }
                        }
                        break;
                    case 4:
                        //周四
                        if (!scheduleTemplate.getThurUsers().isEmpty()){
                            for (String userId : scheduleTemplate.getThurUsers().split(",")) {
                                Schedules schedules = new Schedules();
                                schedules.setPlanId(scheduleTemplate.getPlanId());
                                schedules.setUserId(Long.valueOf(userId));
                                schedules.setBelongingRegionId(scheduleTemplate.getBelongingRegionId());
                                schedules.setForestId(scheduleTemplate.getForestId());
                                schedules.setFarmId(scheduleTemplate.getFarmId());
                                schedules.setFenceId(scheduleTemplate.getFenceId());
                                schedules.setPathId(scheduleTemplate.getPathId());
                                schedules.setDayOfWeek(4);
                                schedules.setDateOfWeek(date);
                                newList.add(schedules);
                            }
                        }
                        break;
                    case 5:
                        //周五
                        if (!scheduleTemplate.getFriUsers().isEmpty()){
                            for (String userId : scheduleTemplate.getFriUsers().split(",")) {
                                Schedules schedules = new Schedules();
                                schedules.setPlanId(scheduleTemplate.getPlanId());
                                schedules.setUserId(Long.valueOf(userId));
                                schedules.setBelongingRegionId(scheduleTemplate.getBelongingRegionId());
                                schedules.setForestId(scheduleTemplate.getForestId());
                                schedules.setFarmId(scheduleTemplate.getFarmId());
                                schedules.setFenceId(scheduleTemplate.getFenceId());
                                schedules.setPathId(scheduleTemplate.getPathId());
                                schedules.setDayOfWeek(5);
                                schedules.setDateOfWeek(date);
                                newList.add(schedules);
                            }
                        }
                        break;
                    case 6:
                        //周六
                        if (!scheduleTemplate.getSatUsers().isEmpty()){
                            for (String userId : scheduleTemplate.getSatUsers().split(",")) {
                                Schedules schedules = new Schedules();
                                schedules.setPlanId(scheduleTemplate.getPlanId());
                                schedules.setUserId(Long.valueOf(userId));
                                schedules.setBelongingRegionId(scheduleTemplate.getBelongingRegionId());
                                schedules.setForestId(scheduleTemplate.getForestId());
                                schedules.setFarmId(scheduleTemplate.getFarmId());
                                schedules.setFenceId(scheduleTemplate.getFenceId());
                                schedules.setPathId(scheduleTemplate.getPathId());
                                schedules.setDayOfWeek(6);
                                schedules.setDateOfWeek(date);
                                newList.add(schedules);
                            }
                        }
                        break;
                    case 7:
                        //周日
                        if (!scheduleTemplate.getSunUsers().isEmpty()){
                            for (String userId : scheduleTemplate.getSunUsers().split(",")) {
                                Schedules schedules = new Schedules();
                                schedules.setPlanId(scheduleTemplate.getPlanId());
                                schedules.setUserId(Long.valueOf(userId));
                                schedules.setBelongingRegionId(scheduleTemplate.getBelongingRegionId());
                                schedules.setForestId(scheduleTemplate.getForestId());
                                schedules.setFarmId(scheduleTemplate.getFarmId());
                                schedules.setFenceId(scheduleTemplate.getFenceId());
                                schedules.setPathId(scheduleTemplate.getPathId());
                                schedules.setDayOfWeek(7);
                                schedules.setDateOfWeek(date);
                                newList.add(schedules);
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
            //批量保存排班信息
            schedulesService.saveBatch(newList);
            result += newList.size();

        }
        if (result > 0) {
            //更新 gen_flag
            schedulePlanMapper.update(null, new UpdateWrapper<SchedulePlan>().eq("plan_id", schedulePlan.getPlanId()).set("gen_flag", 1));
        }
        return result;

    }

}
