package com.tcctyn.iot.forestfire.service.impl;

import com.tcctyn.iot.forestfire.BO.ScopeSimulationBO;
import com.tcctyn.iot.forestfire.domain.SatelliteDataSimulation;
import com.tcctyn.iot.forestfire.mapper.SatelliteDataSimulationMapper;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 临时方案：
 * 暂时采用定时任务模拟生成卫星监测数据
 */
@Service
public class ScheduleTaskService {

    @Resource
    private SatelliteDataSimulationMapper satelliteDataSimulationMapper;

    /**
     * 生成卫星数据，每15分钟生成一波数据
     * 有四个林场，每个林场模拟五个范围，每个范围每十五分钟生成一波数据
     * 模拟数据：温度、湿度、甲烷浓度、经度、纬度
     */
    @Scheduled(fixedDelay = 900000)
    public void generateSatelliteData() {

        //当前时间
        Date currentDate = new Date();

        //温度范围
        double minTemp = 20.0;
        double maxTemp = 100.0;

        //安全温度范围
        double minSafeTemp = 20.0;
        double maxSafeTemp = 35.0;

        //湿度范围（%制）
        double minHumidity = 20.0;
        double maxHumidity = 100.0;

        //安全湿度范围（%制）
        double minSafeHumidity = 30.0;
        double maxSafeHumidity = 60.0;

        //甲烷浓度范围（%制）
        double minMethaneConcentration = 0.0;
        double maxMethaneConcentration = 30.0;

        //安全甲烷浓度范围（%制）
        double minSafeMethaneConcentration = 0.0;
        double maxSafeMethaneConcentration = 1.25;

        //大气压力安全范围
        double minPressure = 80.0;
        double maxPressure = 82.0;

        //风速范围
        double minWindSpeed = 0.0;
        double maxWindSpeed = 2.0;

        //风向范围
        double minWindDirection = 0;
        double maxWindDirection = 360;

        //降雨范围
        double minRainfall = 0.0;
        double maxRainfall = 5.0;

        List<SatelliteDataSimulation> addList = new ArrayList<>(20);

        //给定二十个经纬度范围，四个林场，每个林场取五个范围
        List<ScopeSimulationBO> boList = new ArrayList<>(20);
        //天文台
        boList.add(new ScopeSimulationBO(102.78853, 102.79227, 25.0327, 25.0353));
        boList.add(new ScopeSimulationBO(102.79767, 102.80179, 25.03406, 25.03693));
        boList.add(new ScopeSimulationBO(102.79368, 102.79742, 25.0318, 25.03371));
        boList.add(new ScopeSimulationBO(102.79578, 102.7993, 25.02714, 25.0297));
        boList.add(new ScopeSimulationBO(102.79021, 102.79463, 25.02437, 25.02749));
        //矣六
        boList.add(new ScopeSimulationBO(102.79739, 102.8016, 24.94421, 24.94763));
        boList.add(new ScopeSimulationBO(102.80657, 102.81104, 24.94242, 24.94717));
        boList.add(new ScopeSimulationBO(102.81193, 102.81639, 24.94813, 24.95199));
        boList.add(new ScopeSimulationBO(102.81059, 102.81561, 24.95413, 24.95778));
        boList.add(new ScopeSimulationBO(102.81856, 102.82139, 24.95463, 24.95938));

        //呼马山
        boList.add(new ScopeSimulationBO(102.76823, 102.77098, 25.05649, 25.05906));
        boList.add(new ScopeSimulationBO(102.77189, 102.77589, 25.05466, 25.05754));
        boList.add(new ScopeSimulationBO(102.77138, 102.77428, 25.05756, 25.05999));
        boList.add(new ScopeSimulationBO(102.77415, 102.77646, 25.06004, 25.06205));
        boList.add(new ScopeSimulationBO(102.77282, 102.77599, 25.06264, 25.06388));

        //茶高山
        boList.add(new ScopeSimulationBO(102.81115, 102.81832, 24.97494, 24.97836));
        boList.add(new ScopeSimulationBO(102.81192, 102.81943, 24.97039, 24.97416));
        boList.add(new ScopeSimulationBO(102.81377, 102.82115, 24.96595, 24.96918));
        boList.add(new ScopeSimulationBO(102.81479, 102.82183, 24.9636, 24.96598));
        boList.add(new ScopeSimulationBO(102.81377, 102.81862, 24.9593, 24.96195));

        //如果此前的该区域已经做过降风险
        List<SatelliteDataSimulation> satelliteDataSimulations = satelliteDataSimulationMapper.selectRecentlyData(currentDate);
        List<Integer> riskReducedScopeNoList = satelliteDataSimulations.stream().filter(item -> item.getRiskReduceFlag() == 1)
                .map(item -> item.getRegionScopeNo()).collect(Collectors.toList());


        //循环生成20条数据
        for (int i = 0; i < boList.size(); i++) {
            SatelliteDataSimulation satelliteDataSimulation = new SatelliteDataSimulation();
            satelliteDataSimulation.setTerminalNo("11889");
            satelliteDataSimulation.setUploadTime(currentDate);
            satelliteDataSimulation.setSerialNumber(String.valueOf(i));
            satelliteDataSimulation.setCollectTime(currentDate);

            String temperature = generateRandomTemperature(minTemp, maxTemp);
            String humidity = generateRandomHumidity(minHumidity, maxHumidity);
            String methaneConcentration = generateRandomMethaneConcentration(minMethaneConcentration, maxMethaneConcentration);
            Integer regionScopeNo = i + 1;
            if (!CollectionUtils.isEmpty(riskReducedScopeNoList) && riskReducedScopeNoList.contains(regionScopeNo)) {
                temperature = generateRandomTemperature(minSafeTemp, maxSafeTemp);
                humidity = generateRandomHumidity(minSafeHumidity, maxSafeHumidity);
                methaneConcentration = generateRandomMethaneConcentration(minSafeMethaneConcentration, maxSafeMethaneConcentration);
            }
            satelliteDataSimulation.setTemperature(temperature);
            satelliteDataSimulation.setHumidity(humidity);
            satelliteDataSimulation.setPressure(generateRandomPressure(minPressure, maxPressure));
            satelliteDataSimulation.setWindSpeed(generateRandomWindSpeed(minWindSpeed, maxWindSpeed));
            satelliteDataSimulation.setWindDirection(generateRandomWindDirection(minWindDirection, maxWindDirection));
            satelliteDataSimulation.setRainfall(generateRandomRainfall(minRainfall, maxRainfall));
            satelliteDataSimulation.setMethaneConcentration(methaneConcentration);
            satelliteDataSimulation.setLongitude(generateRandomLongitude(boList.get(i).getMinLongitude(), boList.get(i).getMaxLongitude()));
            satelliteDataSimulation.setLatitude(generateRandomLatitude(boList.get(i).getMinLatitude(), boList.get(i).getMaxLatitude()));
            satelliteDataSimulation.setRegionScopeNo(regionScopeNo);
            satelliteDataSimulation.setRiskLevel(calculateRiskLevel(Double.parseDouble(temperature), Double.parseDouble(humidity), Double.parseDouble(methaneConcentration)));
            satelliteDataSimulation.setRiskReduceFlag(0);
            addList.add(satelliteDataSimulation);
        }
        satelliteDataSimulationMapper.insertBatchSomeColumn(addList);
    }

    /**
     * 生成随机温度
     */
    public String generateRandomTemperature(double minTemp, double maxTemp) {
        double randomTemp = minTemp + (maxTemp - minTemp) * Math.random();
        return String.format("%.2f", randomTemp);
    }

    /**
     * 生成随机湿度
     */
    public String generateRandomHumidity(double minHumidity, double maxHumidity) {
        double randomHumidity = minHumidity + (maxHumidity - minHumidity) * Math.random();
        return String.format("%.2f", randomHumidity);
    }

    /**
     * 生成随机甲烷浓度
     */
    public String generateRandomMethaneConcentration(double minMethaneConcentration, double maxMethaneConcentration) {
        double randomMethaneConcentration = minMethaneConcentration + (maxMethaneConcentration - minMethaneConcentration) * Math.random();
        return String.format("%.2f", randomMethaneConcentration);
    }

    /**
     * 生成随机经度
     */
    public String generateRandomLongitude(double minLongitude, double maxLongitude) {
        double randomLongitude = minLongitude + (maxLongitude - minLongitude) * Math.random();
        return String.format("%.5f", randomLongitude);
    }

    /**
     * 生成随机纬度
     */
    public String generateRandomLatitude(double minLatitude, double maxLatitude) {
        double randomLatitude = minLatitude + (maxLatitude - minLatitude) * Math.random();
        return String.format("%.5f", randomLatitude);
    }

    /**
     * 生成随机大气压力
     */
    public String generateRandomPressure(double minPressure, double maxPressure) {
        double randomPressure = minPressure + (maxPressure - minPressure) * Math.random();
        return String.format("%.2f", randomPressure);
    }

    /**
     * 生成随机风速
     */
    public String generateRandomWindSpeed(double minWindSpeed, double maxWindSpeed) {
        double randomWindSpeed = minWindSpeed + (maxWindSpeed - minWindSpeed) * Math.random();
        return String.format("%.2f", randomWindSpeed);
    }

    /**
     * 生成随机风向
     */
    public String generateRandomWindDirection(double minWindDirection, double maxWindDirection) {
        double randomWindDirection = minWindDirection + (maxWindDirection - minWindDirection) * Math.random();
        return String.format("%.0f", randomWindDirection);
    }

    /**
     * 生成随机雨量
     */
    public String generateRandomRainfall(double minRainfall, double maxRainfall) {
        double randomRainfall = minRainfall + (maxRainfall - minRainfall) * Math.random();
        return String.format("%.2f", randomRainfall);
    }

    /**
     * 计算风险值 (此处计算公式并不严谨，需要注意)
     *
     * @param temperature
     * @param methaneLevel
     * @param humidity
     * @return
     */
    public Integer calculateRiskLevel(double temperature, double humidity, double methaneLevel) {
        if (temperature < 30 || humidity > 70) {
            return 0;
        }
        if (temperature >= 30 && temperature <= 45) {
            if (humidity < 30 || (methaneLevel > 0.1 && methaneLevel <= 1.25)) {
                return 2;
            }
            if (humidity >= 30 && humidity <= 60 || (methaneLevel > 0.1 && methaneLevel <= 1.25)) {
                return 1;
            }
        }
        if (temperature > 45 && temperature < 75) {
            if (humidity < 30 || (methaneLevel > 0.1 && methaneLevel <= 1.25)) {
                return 3;
            }
            if (humidity >= 30 && humidity <= 60 || (methaneLevel > 0.1 && methaneLevel <= 1.25)) {
                return 2;
            }
        }
        if (temperature >= 75 || humidity < 20 || methaneLevel > 5) {
            return 3;
        }
        return 0;
    }
}
