package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.ScheduleTemplateVO;
import com.tcctyn.iot.forestfire.domain.PatrolPath;
import com.tcctyn.iot.forestfire.domain.ScheduleTemplate;
import com.tcctyn.iot.forestfire.mapper.ScheduleTemplateMapper;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IPatrolPathService;
import com.tcctyn.iot.forestfire.service.IScheduleTemplateService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
* <p>
* 排班模板 服务实现类
* </p>
*
* @author liuzh
* @since 2024-10-16
*/
@Service
public class ScheduleTemplateServiceImpl extends ServiceImpl<ScheduleTemplateMapper,ScheduleTemplate> implements IScheduleTemplateService {


    @Resource
    private ScheduleTemplateMapper scheduleTemplateMapper;

    @Resource
    private IPatrolPathService patrolPathService;

    @Resource
    private ICommonService commonService;

    @Override
    public List<ScheduleTemplateVO> list(ScheduleTemplate scheduleTemplate) {
        List<ScheduleTemplate> templateList = scheduleTemplateMapper.selectList(new QueryWrapper<ScheduleTemplate>().eq("plan_id",scheduleTemplate.getPlanId()));

        if(templateList.size() > 0){
            List<ScheduleTemplateVO> templateVOList = new ArrayList<>();
            templateList.forEach(template -> {
                //查询路线id对应的路线名称
                ScheduleTemplateVO scheduleTemplateVO = new ScheduleTemplateVO();
                BeanUtils.copyBeanProp(template,scheduleTemplateVO);
                scheduleTemplateVO.setPathName(patrolPathService.getById(template.getPathId()).getPathName());
                templateVOList.add(scheduleTemplateVO);
            });

            return templateVOList;
        }else {
            //如果排班计划下还没有模板信息，则返回路线列表
            //根据 belongingRegionId 查询路线
            List<PatrolPath> patrolPathList = patrolPathService.selectByRegionId(scheduleTemplate.getBelongingRegionId());
            List<ScheduleTemplateVO> templateVOList = new ArrayList<>();
            patrolPathList.forEach(path -> {
                ScheduleTemplateVO scheduleTemplateVO = new ScheduleTemplateVO();
                scheduleTemplateVO.setPlanId(scheduleTemplate.getPlanId());
                scheduleTemplateVO.setPathId(path.getId());
                scheduleTemplateVO.setPathName(path.getPathName());
                templateVOList.add(scheduleTemplateVO);
            });
            return templateVOList;
        }
    }

    @Override
    public ScheduleTemplate getById(Long id) {
        ScheduleTemplate scheduleTemplate = scheduleTemplateMapper.selectById(id);
        return scheduleTemplate;
    }

    @Override
    public Integer create(ScheduleTemplate scheduleTemplate) {
        return scheduleTemplateMapper.insert(scheduleTemplate);
    }

    @Override
    public Integer delete(Long id) {
        return scheduleTemplateMapper.deleteById(id);
    }

    @Override
    public Integer update(ScheduleTemplate scheduleTemplate) {
        return scheduleTemplateMapper.updateById(scheduleTemplate);
    }

    /**
     * 根据排班计划id获取排班模板
     * @param planId
     * @return
     */
    @Override
    public List<ScheduleTemplate> getByPlanId(Long planId) {
        QueryWrapper<ScheduleTemplate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("plan_id",planId);
        List<ScheduleTemplate> list = scheduleTemplateMapper.selectList(queryWrapper);

        return list;
    }

    /**
     * 批量创建
     * @param scheduleTemplateList
     * @return
     */
    @Override
    public boolean createBatch(List<ScheduleTemplate> scheduleTemplateList) {

        return this.saveOrUpdateBatch(scheduleTemplateList);
    }

}
