package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;

import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.iot.forestfire.VO.WarehouseSelectVO;
import com.tcctyn.iot.forestfire.domain.Warehouse;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.mapper.WarehouseMapper;
import com.tcctyn.iot.forestfire.service.IWarehouseService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 仓库信息 服务实现类
 * </p>
 *
 * @author liuzh
 * @since 2024-10-09
 */
@Service
public class WarehouseServiceImpl extends ServiceImpl<WarehouseMapper, Warehouse> implements IWarehouseService {


    @Resource
    private WarehouseMapper warehouseMapper;

    @Resource
    private RegionInfoMapper regionInfoMapper;

    @Override
    public List<Warehouse> list(Warehouse warehouse) {
        QueryWrapper<Warehouse> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(warehouse);
        return warehouseMapper.selectList(queryWrapper);
    }

    @Override
    public Warehouse getById(Long id) {
        Warehouse warehouse = warehouseMapper.selectById(id);
        return warehouse;
    }

    @Override
    public Integer create(Warehouse warehouse) {
        Long regionId = warehouse.getRegionId();
        if (Objects.nonNull(regionId)) {
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    warehouse.setForestId(regionId);
                    break;
                case 2:
                    warehouse.setForestId(Long.valueOf(ancestorsArr[1]));
                    warehouse.setFarmId(regionId);
                    break;
                case 3:
                    warehouse.setForestId(Long.valueOf(ancestorsArr[1]));
                    warehouse.setFarmId(Long.valueOf(ancestorsArr[2]));
                    warehouse.setFenceId(regionId);
                    break;
                case 4:
                    warehouse.setForestId(Long.valueOf(ancestorsArr[1]));
                    warehouse.setFarmId(Long.valueOf(ancestorsArr[2]));
                    warehouse.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }
        return warehouseMapper.insert(warehouse);
    }

    @Override
    public Integer delete(Long id) {
        return warehouseMapper.deleteById(id);
    }

    @Override
    public Integer update(Warehouse warehouse) {
        //如果修改了 regionId，则需要更新 warehouse 表中的 forestId, farmId, fenceId
        Warehouse originalWarehouse = warehouseMapper.selectById(warehouse.getWarehouseId());
        if (!Objects.equals(originalWarehouse.getRegionId(), warehouse.getRegionId())) {
            Long regionId = warehouse.getRegionId();
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    warehouse.setForestId(regionId);
                    break;
                case 2:
                    warehouse.setForestId(Long.valueOf(ancestorsArr[1]));
                    warehouse.setFarmId(regionId);
                    break;
                case 3:
                    warehouse.setForestId(Long.valueOf(ancestorsArr[1]));
                    warehouse.setFarmId(Long.valueOf(ancestorsArr[2]));
                    warehouse.setFenceId(regionId);
                    break;
                case 4:
                    warehouse.setForestId(Long.valueOf(ancestorsArr[1]));
                    warehouse.setFarmId(Long.valueOf(ancestorsArr[2]));
                    warehouse.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }
        return warehouseMapper.updateById(warehouse);
    }

    @Override
    public List<WarehouseSelectVO> selectWarehouse(Long regionId) {
        List<Warehouse> warehouseList = warehouseMapper.selectList(new QueryWrapper<Warehouse>().eq("region_id", regionId));
        List<WarehouseSelectVO> warehouseSelectVOList = new ArrayList<>();
        for (Warehouse warehouse : warehouseList) {
            WarehouseSelectVO warehouseSelectVO = new WarehouseSelectVO();
            warehouseSelectVO.setWarehouseId(warehouse.getWarehouseId());
            warehouseSelectVO.setWarehouseName(warehouse.getWarehouseName());
            warehouseSelectVO.setLongitude(warehouse.getLongitude());
            warehouseSelectVO.setLatitude(warehouse.getLatitude());
            warehouseSelectVOList.add(warehouseSelectVO);
        }

        return warehouseSelectVOList;

    }

    @Override
    public List<Warehouse> selectAll(Warehouse warehouse) {
        //暂时先支持林区、林场、电子围栏查询，后续有需要可以再加查询条件
        List<Warehouse> warehouseList = warehouseMapper.selectList(new QueryWrapper<Warehouse>().lambda()
                .eq(Objects.nonNull(warehouse.getForestId()), Warehouse::getForestId, warehouse.getForestId())
                .eq(Objects.nonNull(warehouse.getFarmId()), Warehouse::getFarmId, warehouse.getFarmId())
                .eq(Objects.nonNull(warehouse.getFenceId()), Warehouse::getFenceId, warehouse.getFenceId())
        );
        return warehouseList;
    }
}
