package com.tcctyn.iot.hkapi.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import com.tcctyn.iot.hkapi.enums.ResourceCodeEnums;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

@Log4j2
@Data
@Component
public class HkArtemisUtil {
    @Value("${artemis.path}")
    private String ARTEMIS_PATH;
    @Value("${artemis.host}")
    private String ARTEMIS_HOST;
    @Value("${artemis.app-key}")
    private String ARTEMIS_APP_KEY;
    @Value("${artemis.app-secret}")
    private String ARTEMIS_APP_SECRET;

    private String success_code = "0";

    /**
     * 调用POST请求类型接口，这里以获取组织列表为例
     * 接口实际url:https://ip:port/artemis/api/resource/v1/org/orgList
     * <p>
     * api 海康平台接口地址
     * paramMap post请求Form表单参数
     *
     * @return 返回string类型得字符串（里面是json数据）
     */
    public String callPostApiGetOrgList(String api, HashMap<String, Object> paramMap) {
        try {

            /**
             * https://ip:port/artemis/api/resource/v1/org/orgList
             * 通过查阅AI Cloud开放平台文档或网关门户的文档可以看到获取组织列表的接口定义,该接口为POST请求的Rest接口, 入参为JSON字符串，接口协议为https。
             * ArtemisHttpUtil工具类提供了doPostStringArtemis调用POST请求的方法，入参可传JSON字符串, 请阅读开发指南了解方法入参，没有的参数可传null
             */
            ArtemisConfig config = new ArtemisConfig();
            config.setHost(ARTEMIS_HOST); // 代理API网关nginx服务器ip端口
            config.setAppKey(ARTEMIS_APP_KEY);  // 秘钥appkey
            config.setAppSecret(ARTEMIS_APP_SECRET);// 秘钥appSecret

            String getCamsApi = ARTEMIS_PATH + api;

            // 请求参数
            String body = JSON.toJSON(paramMap).toString();
            Map<String, String> path = new HashMap<String, String>(2) {
                {
                    put("https://", getCamsApi);
                }
            };
            Map<String, String> headers = new HashMap<>();
            headers.put("Content-Type", "application/json");

            String result = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, "application/json", headers);
            log.info("调用接口返回结果：{}", result);
            JSONObject resultObj = JSON.parseObject(result);
            if (!success_code.equals(resultObj.get("code"))) {
                throw new Exception("调用失败,错误码为：" + resultObj.getString("code") + ";错误信息：" + ResourceCodeEnums.getValue((Integer) resultObj.get("code")));
            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }

    }
}
